from typing import Callable, Iterator, Sequence
from ..exceptions import DataConversionWarning as DataConversionWarning
from ..utils.extmath import row_norms as row_norms, safe_sparse_dot as safe_sparse_dot
from joblib import effective_n_jobs as effective_n_jobs
from scipy.spatial import distance
from ..metrics import DistanceMetric as DistanceMetric
from ..utils.fixes import sp_version as sp_version, parse_version as parse_version
from ..preprocessing import normalize as normalize
from scipy.sparse import csr_matrix as csr_matrix, issparse as issparse
from ..utils.parallel import delayed as delayed, Parallel as Parallel
from .. import config_context as config_context
from ..utils.validation import check_non_negative as check_non_negative
from ..gaussian_process.kernels import ExpSineSquared
from numpy import ndarray
from functools import partial as partial
from ..utils import (
    check_array as check_array,
    gen_even_slices as gen_even_slices,
    gen_batches as gen_batches,
    get_chunk_n_rows as get_chunk_n_rows,
    is_scalar_nan as is_scalar_nan,
)
from ..gaussian_process.kernels import Kernel as GPKernel
from ._pairwise_distances_reduction import ArgKmin as ArgKmin
from .._typing import MatrixLike, ArrayLike, Int, Float

# Authors: Alexandre Gramfort <alexandre.gramfort@inria.fr>
#          Mathieu Blondel <mathieu@mblondel.org>
#          Robert Layton <robertlayton@gmail.com>
#          Andreas Mueller <amueller@ais.uni-bonn.de>
#          Philippe Gervais <philippe.gervais@inria.fr>
#          Lars Buitinck
#          Joel Nothman <joel.nothman@gmail.com>
# License: BSD 3 clause

import itertools
import warnings

import numpy as np


def check_pairwise_arrays(
    X: MatrixLike,
    Y: None | MatrixLike,
    *,
    precomputed: bool = False,
    dtype: None | Sequence[type] | str | type = None,
    accept_sparse: Sequence[str] | str | bool = "csr",
    force_all_finite: str | bool = True,
    copy: bool = False,
) -> tuple[ndarray, ndarray]:
    ...


def check_paired_arrays(X: MatrixLike, Y: MatrixLike) -> tuple[ndarray, ndarray]:
    ...


# Pairwise distances
def euclidean_distances(
    X: MatrixLike,
    Y: None | MatrixLike = None,
    *,
    Y_norm_squared: None | MatrixLike | ArrayLike = None,
    squared: bool = False,
    X_norm_squared: None | MatrixLike | ArrayLike = None,
) -> ndarray:
    ...


def nan_euclidean_distances(
    X: MatrixLike,
    Y: None | MatrixLike = None,
    *,
    squared: bool = False,
    missing_values: float | int = ...,
    copy: bool = True,
) -> ndarray:
    ...


def pairwise_distances_argmin_min(
    X: MatrixLike,
    Y: MatrixLike,
    *,
    axis: Int = 1,
    metric: str | Callable = "euclidean",
    metric_kwargs: None | dict = None,
) -> tuple[ndarray, ndarray]:
    ...


def pairwise_distances_argmin(
    X: MatrixLike,
    Y: MatrixLike,
    *,
    axis: Int = 1,
    metric: str | Callable = "euclidean",
    metric_kwargs: None | dict = None,
) -> ndarray:
    ...


def haversine_distances(X: MatrixLike, Y: None | MatrixLike = None) -> ndarray:
    ...


def manhattan_distances(
    X: MatrixLike,
    Y: None | MatrixLike = None,
    *,
    sum_over_features: str | bool = "deprecated",
) -> ndarray:
    ...


def cosine_distances(X: MatrixLike, Y: None | MatrixLike = None) -> ndarray:
    ...


# Paired distances
def paired_euclidean_distances(X: MatrixLike, Y: MatrixLike) -> ndarray:
    ...


def paired_manhattan_distances(X: MatrixLike, Y: MatrixLike) -> ndarray:
    ...


def paired_cosine_distances(X: MatrixLike, Y: MatrixLike) -> ndarray:
    ...


PAIRED_DISTANCES: dict = ...


def paired_distances(
    X: ArrayLike, Y: ArrayLike, *, metric: str | Callable = "euclidean", **kwds
) -> ndarray:
    ...


# Kernels
def linear_kernel(
    X: MatrixLike, Y: None | MatrixLike = None, dense_output: bool = True
) -> ndarray:
    ...


def polynomial_kernel(
    X: MatrixLike,
    Y: None | MatrixLike = None,
    degree: Int = 3,
    gamma: None | Float = None,
    coef0: Float = 1,
) -> ndarray:
    ...


def sigmoid_kernel(
    X: MatrixLike,
    Y: None | MatrixLike = None,
    gamma: None | Float = None,
    coef0: Float = 1,
) -> ndarray:
    ...


def rbf_kernel(
    X: MatrixLike, Y: None | MatrixLike = None, gamma: None | Float = None
) -> ndarray:
    ...


def laplacian_kernel(
    X: MatrixLike, Y: None | MatrixLike = None, gamma: None | Float = None
) -> ndarray:
    ...


def cosine_similarity(
    X: MatrixLike, Y: None | MatrixLike = None, dense_output: bool = True
) -> ndarray:
    ...


def additive_chi2_kernel(X: MatrixLike, Y: None | MatrixLike = None) -> ndarray:
    ...


def chi2_kernel(
    X: MatrixLike, Y: None | MatrixLike = None, gamma: Float = 1.0
) -> ndarray:
    ...


# Helper functions - distance
PAIRWISE_DISTANCE_FUNCTIONS: dict = ...


def distance_metrics() -> dict:
    ...


_VALID_METRICS: list = ...

_NAN_METRICS: list = ...


def pairwise_distances_chunked(
    X: MatrixLike,
    Y: None | MatrixLike = None,
    *,
    reduce_func: None | Callable = None,
    metric: str | Callable = "euclidean",
    n_jobs: None | Int = None,
    working_memory: None | Int = None,
    **kwds,
) -> Iterator[None | ndarray | tuple[ndarray, ndarray]]:
    ...


def pairwise_distances(
    X: MatrixLike,
    Y: None | MatrixLike = None,
    metric: str | Callable = "euclidean",
    *,
    n_jobs: None | Int = None,
    force_all_finite: str | bool = True,
    **kwds,
) -> ndarray:
    ...


# These distances require boolean arrays, when using scipy.spatial.distance
PAIRWISE_BOOLEAN_FUNCTIONS: list = ...

# Helper functions - distance
PAIRWISE_KERNEL_FUNCTIONS: dict = ...


def kernel_metrics() -> dict:
    ...


KERNEL_PARAMS: dict = ...


def pairwise_kernels(
    X: MatrixLike,
    Y: None | MatrixLike = None,
    metric: ExpSineSquared | str | Callable = "linear",
    *,
    filter_params: bool = False,
    n_jobs: None | Int = None,
    **kwds,
) -> ndarray:
    ...



from typing import Any, Callable

from django.http.request import HttpRequest
from django.http.response import HttpResponse, HttpResponseBase

logger: Any

class BaseHandler:
    def load_middleware(self) -> None: ...
    def make_view_atomic(self, view: Callable[..., Any]) -> Callable[..., Any]: ...
    def get_exception_response(
        self, request: Any, resolver: Any, status_code: Any, exception: Any
    ) -> Any: ...
    def get_response(self, request: HttpRequest) -> HttpResponseBase: ...
    def process_exception_by_middleware(
        self, exception: Exception, request: HttpRequest
    ) -> HttpResponse: ...



from numpy import ndarray
from numpy.typing import NDArray
import numpy as np

from .._shared.utils import deprecate_kwarg, warn

from . import grayreconstruct, _util

def _add_constant_clip(image, const_value): ...
def _subtract_constant_clip(image, const_value): ...
@deprecate_kwarg(
    kwarg_mapping={"selem": "footprint"},
    removed_version="1.0",
    deprecated_version="0.19",
)
def h_maxima(image: NDArray, h, footprint: NDArray | None = None) -> NDArray: ...
@deprecate_kwarg(
    kwarg_mapping={"selem": "footprint"},
    removed_version="1.0",
    deprecated_version="0.19",
)
def h_minima(image: NDArray, h, footprint: NDArray | None = None) -> NDArray: ...
@deprecate_kwarg(
    kwarg_mapping={"selem": "footprint"},
    removed_version="1.0",
    deprecated_version="0.19",
)
def local_maxima(
    image: NDArray,
    footprint: NDArray | None = None,
    connectivity: int | None = None,
    indices: bool = False,
    allow_borders: bool = True,
) -> np.ndarray | tuple[np.ndarray]: ...
@deprecate_kwarg(
    kwarg_mapping={"selem": "footprint"},
    removed_version="1.0",
    deprecated_version="0.19",
)
def local_minima(
    image: NDArray,
    footprint: NDArray | None = None,
    connectivity: int | None = None,
    indices: bool = False,
    allow_borders: bool = True,
) -> np.ndarray | tuple[np.ndarray]: ...



from collections.abc import Iterable
from typing import Mapping
from numpy.typing import ArrayLike

import itertools

from ...classes.graph import Graph

__all__ = [
    "clustering",
    "average_clustering",
    "latapy_clustering",
    "robins_alexander_clustering",
]

def cc_dot(nu, nv): ...
def cc_max(nu, nv): ...
def cc_min(nu, nv): ...

modes: dict = ...

def latapy_clustering(G: Graph, nodes=None, mode: str = "dot") -> Mapping: ...

clustering = ...

def average_clustering(
    G: Graph, nodes: ArrayLike | Iterable | None = None, mode: str = "dot"
) -> float: ...
def robins_alexander_clustering(G: Graph) -> float: ...



from typing import Any, Callable, ClassVar, Literal, TypeVar
from ..decomposition import KernelPCA as KernelPCA
from scipy.sparse._csr import csr_matrix
from ..preprocessing import KernelCenterer as KernelCenterer
from scipy.sparse.csgraph import (
    shortest_path as shortest_path,
    connected_components as connected_components,
)
from ..neighbors._ball_tree import BallTree
from ..neighbors._kd_tree import KDTree
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from numpy import ndarray
from numbers import Integral as Integral, Real as Real
from ..neighbors import (
    NearestNeighbors,
    kneighbors_graph as kneighbors_graph,
    radius_neighbors_graph as radius_neighbors_graph,
)
from ..base import BaseEstimator, TransformerMixin, ClassNamePrefixFeaturesOutMixin
from scipy.sparse import issparse as issparse
from .._typing import Float, Int, ArrayLike, MatrixLike
from ..utils.validation import check_is_fitted as check_is_fitted

Isomap_Self = TypeVar("Isomap_Self", bound="Isomap")


# Author: Jake Vanderplas  -- <vanderplas@astro.washington.edu>
# License: BSD 3 clause (C) 2011
import warnings

import numpy as np


class Isomap(ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    dist_matrix_: ArrayLike = ...
    nbrs_: NearestNeighbors = ...
    kernel_pca_: Any = ...
    embedding_: ArrayLike = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        n_neighbors: None | int = 5,
        radius: None | Float = None,
        n_components: Int = 2,
        eigen_solver: Literal["auto", "arpack", "dense", "auto"] = "auto",
        tol: Float = 0,
        max_iter: None | Int = None,
        path_method: Literal["auto", "FW", "D", "auto"] = "auto",
        neighbors_algorithm: Literal[
            "auto", "brute", "kd_tree", "ball_tree", "auto"
        ] = "auto",
        n_jobs: None | int = None,
        metric: str | Callable = "minkowski",
        p: Int = 2,
        metric_params: None | dict = None,
    ) -> None:
        ...

    def reconstruction_error(self) -> float:
        ...

    def fit(
        self: Isomap_Self,
        X: ArrayLike | BallTree | NearestNeighbors | KDTree | csr_matrix,
        y: Any = None,
    ) -> Isomap_Self:
        ...

    def fit_transform(self, X: BallTree | ArrayLike | KDTree, y: Any = None) -> ndarray:
        ...

    def transform(self, X: MatrixLike) -> ndarray:
        ...



from numpy.typing import ArrayLike
from itertools import zip_longest

__all__ = ["union_all", "compose_all", "disjoint_union_all", "intersection_all"]

def union_all(graphs, rename: bool = ...): ...
def disjoint_union_all(graphs: ArrayLike): ...
def compose_all(graphs: ArrayLike): ...
def intersection_all(graphs: ArrayLike): ...



"""Startup for Python.

Avoid activating Rich features that break functionality outside of the REPL.
"""


def main():
    from rich import inspect, traceback  # noqa: F401  # pyright: ignore

    traceback.install()

    if not is_notebook_or_ipython():
        from rich import pretty, print  # noqa: F401  # pyright: ignore

        pretty.install()


# https://stackoverflow.com/a/39662359
def is_notebook_or_ipython() -> bool:
    try:
        shell = get_ipython().__class__.__name__  # pyright: ignore  # Dynamic
    except NameError:
        return False  # Probably standard Python interpreter
    else:
        return shell == "TerminalInteractiveShell"


main()



"""
This type stub file was generated by pyright.
"""

from IPython.core import magic_arguments
from IPython.core.magic import Magics, line_magic, magics_class
from IPython.testing.skipdoctest import skip_doctest

"""Implementation of basic magic functions."""
class MagicsDisplay:
    def __init__(self, magics_manager, ignore=...) -> None:
        ...
    
    def __str__(self) -> str:
        ...
    


@magics_class
class BasicMagics(Magics):
    """Magics that provide central IPython functionality.

    These are various magics that don't fit into specific categories but that
    are all part of the base 'IPython experience'."""
    @skip_doctest
    @magic_arguments.magic_arguments()
    @magic_arguments.argument('-l', '--line', action='store_true', help="""Create a line magic alias.""")
    @magic_arguments.argument('-c', '--cell', action='store_true', help="""Create a cell magic alias.""")
    @magic_arguments.argument('name', help="""Name of the magic to be created.""")
    @magic_arguments.argument('target', help="""Name of the existing line or cell magic.""")
    @magic_arguments.argument('-p', '--params', default=None, help="""Parameters passed to the magic function.""")
    @line_magic
    def alias_magic(self, line=...): # -> None:
        """Create an alias for an existing line or cell magic.

        Examples
        --------
        ::

          In [1]: %alias_magic t timeit
          Created `%t` as an alias for `%timeit`.
          Created `%%t` as an alias for `%%timeit`.

          In [2]: %t -n1 pass
          1 loops, best of 3: 954 ns per loop

          In [3]: %%t -n1
             ...: pass
             ...:
          1 loops, best of 3: 954 ns per loop

          In [4]: %alias_magic --cell whereami pwd
          UsageError: Cell magic function `%%pwd` not found.
          In [5]: %alias_magic --line whereami pwd
          Created `%whereami` as an alias for `%pwd`.

          In [6]: %whereami
          Out[6]: u'/home/testuser'

          In [7]: %alias_magic h history "-p -l 30" --line
          Created `%h` as an alias for `%history -l 30`.
        """
        ...
    
    @line_magic
    def lsmagic(self, parameter_s=...): # -> MagicsDisplay:
        """List currently available magic functions."""
        ...
    
    @line_magic
    def magic(self, parameter_s=...): # -> None:
        """Print information about the magic function system.

        Supported formats: -latex, -brief, -rest
        """
        ...
    
    @line_magic
    def page(self, parameter_s=...): # -> None:
        """Pretty print the object and display it through a pager.

        %page [options] OBJECT

        If no object is given, use _ (last output).

        Options:

          -r: page str(object), don't pretty-print it."""
        ...
    
    @line_magic
    def pprint(self, parameter_s=...): # -> None:
        """Toggle pretty printing on/off."""
        ...
    
    @line_magic
    def colors(self, parameter_s=...): # -> None:
        """Switch color scheme for prompts, info system and exception handlers.

        Currently implemented schemes: NoColor, Linux, LightBG.

        Color scheme names are not case-sensitive.

        Examples
        --------
        To get a plain black and white terminal::

          %colors nocolor
        """
        ...
    
    @line_magic
    def xmode(self, parameter_s=...): # -> None:
        """Switch modes for the exception handlers.

        Valid modes: Plain, Context, Verbose, and Minimal.

        If called without arguments, acts as a toggle.

        When in verbose mode the value `--show` (and `--hide`)
        will respectively show (or hide) frames with ``__tracebackhide__ =
        True`` value set.
        """
        ...
    
    @line_magic
    def quickref(self, arg): # -> None:
        """ Show a quick reference sheet """
        ...
    
    @line_magic
    def doctest_mode(self, parameter_s=...): # -> None:
        """Toggle doctest mode on and off.

        This mode is intended to make IPython behave as much as possible like a
        plain Python shell, from the perspective of how its prompts, exceptions
        and output look.  This makes it easy to copy and paste parts of a
        session into doctests.  It does so by:

        - Changing the prompts to the classic ``>>>`` ones.
        - Changing the exception reporting mode to 'Plain'.
        - Disabling pretty-printing of output.

        Note that IPython also supports the pasting of code snippets that have
        leading '>>>' and '...' prompts in them.  This means that you can paste
        doctests from files or docstrings (even if they have leading
        whitespace), and the code will execute correctly.  You can then use
        '%history -t' to see the translated history; this will give you the
        input after removal of all the leading prompts and whitespace, which
        can be pasted back into an editor.

        With these features, you can switch into this mode easily whenever you
        need to do testing and changes to doctests, without having to leave
        your existing IPython session.
        """
        ...
    
    @line_magic
    def gui(self, parameter_s=...): # -> None:
        """Enable or disable IPython GUI event loop integration.

        %gui [GUINAME]

        This magic replaces IPython's threaded shells that were activated
        using the (pylab/wthread/etc.) command line flags.  GUI toolkits
        can now be enabled at runtime and keyboard
        interrupts should work without any problems.  The following toolkits
        are supported:  wxPython, PyQt4, PyGTK, Tk and Cocoa (OSX)::

            %gui wx      # enable wxPython event loop integration
            %gui qt      # enable PyQt/PySide event loop integration
                         # with the latest version available.
            %gui qt6     # enable PyQt6/PySide6 event loop integration
            %gui qt5     # enable PyQt5/PySide2 event loop integration
            %gui gtk     # enable PyGTK event loop integration
            %gui gtk3    # enable Gtk3 event loop integration
            %gui gtk4    # enable Gtk4 event loop integration
            %gui tk      # enable Tk event loop integration
            %gui osx     # enable Cocoa event loop integration
                         # (requires %matplotlib 1.1)
            %gui         # disable all event loop integration

        WARNING:  after any of these has been called you can simply create
        an application object, but DO NOT start the event loop yourself, as
        we have already handled that.
        """
        ...
    
    @skip_doctest
    @line_magic
    def precision(self, s=...):
        """Set floating point precision for pretty printing.

        Can set either integer precision or a format string.

        If numpy has been imported and precision is an int,
        numpy display precision will also be set, via ``numpy.set_printoptions``.

        If no argument is given, defaults will be restored.

        Examples
        --------
        ::

            In [1]: from math import pi

            In [2]: %precision 3
            Out[2]: u'%.3f'

            In [3]: pi
            Out[3]: 3.142

            In [4]: %precision %i
            Out[4]: u'%i'

            In [5]: pi
            Out[5]: 3

            In [6]: %precision %e
            Out[6]: u'%e'

            In [7]: pi**10
            Out[7]: 9.364805e+04

            In [8]: %precision
            Out[8]: u'%r'

            In [9]: pi**10
            Out[9]: 93648.047476082982
        """
        ...
    
    @magic_arguments.magic_arguments()
    @magic_arguments.argument('filename', type=str, help='Notebook name or filename')
    @line_magic
    def notebook(self, s): # -> None:
        """Export and convert IPython notebooks.

        This function can export the current IPython history to a notebook file.
        For example, to export the history to "foo.ipynb" do "%notebook foo.ipynb".
        """
        ...
    


@magics_class
class AsyncMagics(BasicMagics):
    @line_magic
    def autoawait(self, parameter_s): # -> None:
        """
        Allow to change the status of the autoawait option.

        This allow you to set a specific asynchronous code runner.

        If no value is passed, print the currently used asynchronous integration
        and whether it is activated.

        It can take a number of value evaluated in the following order:

        - False/false/off deactivate autoawait integration
        - True/true/on activate autoawait integration using configured default
          loop
        - asyncio/curio/trio activate autoawait integration and use integration
          with said library.

        - `sync` turn on the pseudo-sync integration (mostly used for
          `IPython.embed()` which does not run IPython with a real eventloop and
          deactivate running asynchronous code. Turning on Asynchronous code with
          the pseudo sync loop is undefined behavior and may lead IPython to crash.

        If the passed parameter does not match any of the above and is a python
        identifier, get said object from user namespace and set it as the
        runner, and activate autoawait.

        If the object is a fully qualified object name, attempt to import it and
        set it as the runner, and activate autoawait.

        The exact behavior of autoawait is experimental and subject to change
        across version of IPython and Python.
        """
        ...
    





from collections.abc import Iterable
from typing import Any, Callable
import html.entities as htmlentitydefs
import re
import warnings
from ast import literal_eval
from collections import defaultdict
from enum import Enum
from io import StringIO
from typing import Any, NamedTuple

from ..classes.graph import Graph
from ..exception import NetworkXError
from ..utils import open_file

__all__ = ["read_gml", "parse_gml", "generate_gml", "write_gml"]

def escape(text): ...
def unescape(text): ...
def literal_destringizer(rep: str) -> Any: ...
def read_gml(path, label: str = "label", destringizer: Callable | None = None): ...
def parse_gml(
    lines: str | Iterable[str],
    label: str = "label",
    destringizer: Callable | None = None,
): ...

class Pattern(Enum):

    KEYS = ...
    REALS = ...
    INTS = ...
    STRINGS = ...
    DICT_START = ...
    DICT_END = ...
    COMMENT_WHITESPACE = ...

class Token(NamedTuple):
    category: Pattern = ...
    value: Any = ...
    line: int = ...
    position: int = ...

LIST_START_VALUE: str = ...

def parse_gml_lines(lines, label, destringizer): ...
def literal_stringizer(value: Any) -> str: ...
def generate_gml(G: Graph, stringizer: Callable | None = None): ...
def write_gml(G: Graph, path, stringizer: Callable | None = None): ...



from typing import Literal
from ._typing import *
from .transforms import Affine2D, Transform
from .path import Path
from ._enums import CapStyle, JoinStyle

class MarkerStyle:

    markers: list = ...
    filled_markers: list = ...
    fillstyles: list = ...

    def __init__(
        self,
        marker: str | ArrayLike | Path | MarkerStyle | None = ...,
        fillstyle: Literal["full", "left", "right", "bottom", "top", "none"] = ...,
        transform: Transform = ...,
        capstyle: CapStyle = ...,
        joinstyle: JoinStyle = ...,
    ) -> None: ...
    def __bool__(self) -> bool: ...
    def is_filled(self) -> bool: ...
    def get_fillstyle(self) -> str: ...
    def get_joinstyle(self) -> JoinStyle: ...
    def get_capstyle(self) -> CapStyle: ...
    def get_marker(self): ...
    def get_path(self) -> Path: ...
    def get_transform(self) -> Transform: ...
    def get_alt_path(self) -> Path: ...
    def get_alt_transform(self) -> Transform: ...
    def get_snap_threshold(self): ...
    def get_user_transform(self): ...
    def transformed(self, transform: Affine2D | None = ...): ...
    def rotated(self, *, deg: float = ..., rad: float = ...): ...
    def scaled(self, sx: float, sy: float = ...): ...



from typing import (
    Any,
    Callable,
    Generic,
    Iterable,
    List,
    Optional,
    Tuple,
    TypeVar,
    Union,
    overload,
)

from django.db.models.expressions import Combinable
from django.db.models.fields import Field, _ErrorMessagesToOverride, _ValidatorCallable
from typing_extensions import Literal

from .mixins import CheckFieldDefaultMixin

_V = TypeVar("_V", bound=Optional[Any])

class ArrayField(
    CheckFieldDefaultMixin,
    Generic[_V],
    Field[Union[_V, Combinable], _V],
):
    empty_strings_allowed: bool = ...
    default_error_messages: Any = ...
    size: Optional[int] = ...
    default_validators: Any = ...
    from_db_value: Any = ...
    base_field: Field[_V, _V] = ...
    @overload
    def __new__(  # type: ignore [misc]
        cls,
        base_field: Field[Any, _V],
        size: Optional[int] = ...,
        verbose_name: Optional[str] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: Literal[False] = ...,
        db_index: bool = ...,
        default: Optional[Union[List[_V], Callable[[], List[_V]]]] = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Iterable[
            Union[Tuple[List[_V], str], Tuple[str, Iterable[Tuple[List[_V], str]]]]
        ] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ) -> ArrayField[List[_V]]: ...
    @overload
    def __new__(
        cls,
        base_field: Field[Any, _V],
        size: Optional[int] = ...,
        verbose_name: Optional[str] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: Literal[True] = ...,
        db_index: bool = ...,
        default: Optional[Union[List[_V], Callable[[], List[_V]]]] = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Iterable[
            Union[Tuple[List[_V], str], Tuple[str, Iterable[Tuple[List[_V], str]]]]
        ] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ) -> ArrayField[Optional[List[_V]]]: ...
    @property
    def description(self) -> str: ...  # type: ignore [override]
    def get_transform(self, name: Any) -> Any: ...



from .node_link import node_link_data as node_link_data, node_link_graph as node_link_graph
from .adjacency import adjacency_data as adjacency_data, adjacency_graph as adjacency_graph
from .tree import tree_data as tree_data, tree_graph as tree_graph
from .jit import jit_graph as jit_graph, jit_data as jit_data
from .cytoscape import cytoscape_data as cytoscape_data, cytoscape_graph as cytoscape_graph



import itertools
import sys
from heapq import heapify, heappop, heappush
from ...classes.graph import Graph
from ...utils import not_implemented_for

__all__ = ["treewidth_min_degree", "treewidth_min_fill_in"]

def treewidth_min_degree(G: Graph): ...
def treewidth_min_fill_in(G: Graph): ...

class MinDegreeHeuristic:
    def __init__(self, graph): ...
    def best_node(self, graph): ...

def min_fill_in_heuristic(graph): ...
def treewidth_decomp(G: Graph, heuristic=...): ...



from vispy.util.svg.color import Color
from numpy.typing import ArrayLike, NDArray

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

from ..geometry import create_box
from .mesh import MeshVisual
from .visual import CompoundVisual

class BoxVisual(CompoundVisual):
    def __init__(
        self,
        width: float = 1,
        height: float = 1,
        depth: float = 1,
        width_segments: int = 1,
        height_segments: float = 1,
        depth_segments: float = 1,
        planes: ArrayLike | None = None,
        vertex_colors: NDArray | None = None,
        face_colors: NDArray | None = None,
        color: Color = ...,
        edge_color: tuple | Color | None = None,
        **kwargs,
    ): ...
    @property
    def mesh(self): ...
    @mesh.setter
    def mesh(self, mesh): ...
    @property
    def border(self): ...
    @border.setter
    def border(self, border): ...



import os
import numpy as np

# Putting this in cython was giving strange bugs for different versions
# of cython which seemed to indicate troubles with the __file__ variable
# not being defined. Keeping it in pure python makes it more reliable
this_dir = ...
POS = ...
POS0 = ...
POS1 = ...



from typing import Literal, Sequence
from string import Template as Template
from contextlib import closing as closing
from ..base import BaseEstimator
from io import StringIO as StringIO
from inspect import isclass as isclass
from .. import config_context as config_context
import html


class _IDCounter:
    def __init__(self, prefix: str) -> None:
        ...

    def get_id(self):
        ...


_CONTAINER_ID_COUNTER = ...
_ESTIMATOR_ID_COUNTER = ...


class _VisualBlock:
    def __init__(
        self,
        kind: Literal["serial", "parallel", "single"],
        estimators: Sequence[BaseEstimator] | BaseEstimator | Sequence[_VisualBlock],
        *,
        names: None | Sequence[str] = None,
        name_details: Sequence[str] | None | str = None,
        dash_wrapped: bool = True
    ) -> None:
        ...


_STYLE = ...  # noqa


def estimator_html_repr(estimator: BaseEstimator) -> str:
    ...



from typing import Callable, ClassVar, Literal, TypeVar
from ._base import NeighborsBase, KNeighborsMixin, RadiusNeighborsMixin
from numpy import ndarray
from ..utils.extmath import weighted_mode as weighted_mode
from ..utils._param_validation import StrOptions as StrOptions
from numbers import Integral as Integral
from ..base import ClassifierMixin
from .._typing import Int, MatrixLike, ArrayLike, Float

RadiusNeighborsClassifier_Self = TypeVar(
    "RadiusNeighborsClassifier_Self", bound="RadiusNeighborsClassifier"
)
KNeighborsClassifier_Self = TypeVar(
    "KNeighborsClassifier_Self", bound="KNeighborsClassifier"
)


import numpy as np

import warnings


class KNeighborsClassifier(KNeighborsMixin, ClassifierMixin, NeighborsBase):
    outputs_2d_: bool = ...
    n_samples_fit_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    effective_metric_params_: dict = ...
    effective_metric_: str | Callable = ...
    classes_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_neighbors: Int = 5,
        *,
        weights: None
        | Literal["uniform", "distance", "uniform"]
        | Callable = "uniform",
        algorithm: Literal["auto", "ball_tree", "kd_tree", "brute", "auto"] = "auto",
        leaf_size: Int = 30,
        p: Int = 2,
        metric: str | Callable = "minkowski",
        metric_params: None | dict = None,
        n_jobs: None | Int = None,
    ) -> None:
        ...

    def fit(
        self: KNeighborsClassifier_Self, X: MatrixLike, y: MatrixLike | ArrayLike
    ) -> KNeighborsClassifier_Self:
        ...

    def predict(self, X: MatrixLike) -> ndarray:
        ...

    def predict_proba(self, X: MatrixLike) -> ndarray | list[ndarray]:
        ...


class RadiusNeighborsClassifier(RadiusNeighborsMixin, ClassifierMixin, NeighborsBase):
    outputs_2d_: bool = ...
    outlier_label_: ArrayLike | int = ...
    n_samples_fit_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    effective_metric_params_: dict = ...
    effective_metric_: str | Callable = ...
    classes_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        radius: Float = 1.0,
        *,
        weights: None
        | Literal["uniform", "distance", "uniform"]
        | Callable = "uniform",
        algorithm: Literal["auto", "ball_tree", "kd_tree", "brute", "auto"] = "auto",
        leaf_size: Int = 30,
        p: Int = 2,
        metric: str | Callable = "minkowski",
        outlier_label: None | str = None,
        metric_params: None | dict = None,
        n_jobs: None | Int = None,
    ) -> None:
        ...

    def fit(
        self: RadiusNeighborsClassifier_Self, X: MatrixLike, y: MatrixLike | ArrayLike
    ) -> RadiusNeighborsClassifier_Self:
        ...

    def predict(self, X: MatrixLike) -> ndarray:
        ...

    def predict_proba(self, X: MatrixLike) -> ndarray:
        ...



from .clustering_coefficient import average_clustering as average_clustering
from ...algorithms.approximation.clique import (
    clique_removal as clique_removal,
    max_clique as max_clique,
    large_clique_size as large_clique_size,
    maximum_independent_set as maximum_independent_set,
)

from .connectivity import (
    local_node_connectivity as local_node_connectivity,
    node_connectivity as node_connectivity,
    all_pairs_node_connectivity as all_pairs_node_connectivity,
)

from .distance_measures import diameter as diameter

from .dominating_set import (
    min_weighted_dominating_set as min_weighted_dominating_set,
    min_edge_dominating_set as min_edge_dominating_set,
)

from .kcomponents import k_components as k_components
from .matching import min_maximal_matching as min_maximal_matching
from .ramsey import ramsey_R2 as ramsey_R2
from .steinertree import metric_closure as metric_closure, steiner_tree as steiner_tree

from .traveling_salesman import (
    traveling_salesman_problem as traveling_salesman_problem,
    christofides as christofides,
    asadpour_atsp as asadpour_atsp,
    greedy_tsp as greedy_tsp,
    simulated_annealing_tsp as simulated_annealing_tsp,
    threshold_accepting_tsp as threshold_accepting_tsp,
)

from .treewidth import (
    treewidth_min_degree as treewidth_min_degree,
    treewidth_min_fill_in as treewidth_min_fill_in,
)

from .vertex_cover import min_weighted_vertex_cover as min_weighted_vertex_cover
from .maxcut import (
    randomized_partitioning as randomized_partitioning,
    one_exchange as one_exchange,
)



"""
This type stub file was generated by pyright.
"""

"""
A simple utility to import something by its string name.
"""
def import_item(name):
    """Import and return ``bar`` given the string ``foo.bar``.

    Calling ``bar = import_item("foo.bar")`` is the functional equivalent of
    executing the code ``from foo import bar``.

    Parameters
    ----------
    name : string
        The fully qualified name of the module/package being imported.

    Returns
    -------
    mod : module object
        The module that was imported.
    """
    ...




import datetime
from io import BytesIO
from json import JSONEncoder
from typing import (
    Any,
    AsyncIterable,
    Dict,
    Iterable,
    Iterator,
    List,
    Optional,
    Tuple,
    Type,
    Union,
    overload,
)

from django.core.handlers.wsgi import WSGIRequest
from django.http.cookie import SimpleCookie
from django.template import Context, Template
from django.test.client import Client
from django.urls import ResolverMatch

class BadHeaderError(ValueError): ...

class HttpResponseBase(Iterable[Any]):
    status_code: int = ...
    cookies: SimpleCookie[str] = ...
    reason_phrase: str = ...
    charset: str = ...
    closed: bool = ...
    def __init__(
        self,
        content_type: Optional[str] = ...,
        status: Optional[int] = ...,
        reason: Optional[str] = ...,
        charset: Optional[str] = ...,
    ) -> None: ...
    def serialize_headers(self) -> bytes: ...
    def __setitem__(
        self, header: Union[str, bytes], value: Union[str, bytes, int]
    ) -> None: ...
    def __delitem__(self, header: Union[str, bytes]) -> None: ...
    def __getitem__(self, header: Union[str, bytes]) -> str: ...
    def has_header(self, header: str) -> bool: ...
    def items(self) -> Iterable[Tuple[str, str]]: ...
    @overload
    def get(self, header: Union[str, bytes], alternate: Optional[str]) -> str: ...
    @overload
    def get(self, header: Union[str, bytes]) -> Optional[str]: ...
    def set_cookie(
        self,
        key: str,
        value: str = ...,
        max_age: Optional[int] = ...,
        expires: Optional[Union[str, datetime.datetime]] = ...,
        path: str = ...,
        domain: Optional[str] = ...,
        secure: bool = ...,
        httponly: bool = ...,
        samesite: str = ...,
    ) -> None: ...
    def setdefault(self, key: str, value: str) -> None: ...
    def set_signed_cookie(
        self, key: str, value: str, salt: str = ..., **kwargs: Any
    ) -> None: ...
    def delete_cookie(
        self, key: str, path: str = ..., domain: Optional[str] = ...
    ) -> None: ...
    def make_bytes(self, value: object) -> bytes: ...
    def close(self) -> None: ...
    def write(self, content: Union[str, bytes]) -> None: ...
    def flush(self) -> None: ...
    def tell(self) -> int: ...
    def readable(self) -> bool: ...
    def seekable(self) -> bool: ...
    def writable(self) -> bool: ...
    def writelines(self, lines: Iterable[object]) -> Any: ...
    def __iter__(self) -> Iterator[Any]: ...

class HttpResponse(HttpResponseBase):
    content: Any
    csrf_cookie_set: bool
    redirect_chain: List[Tuple[str, int]]
    sameorigin: bool
    test_server_port: str
    test_was_secure_request: bool
    xframe_options_exempt: bool
    streaming: bool = ...
    def __init__(self, content: object = ..., *args: Any, **kwargs: Any) -> None: ...
    def serialize(self) -> bytes: ...
    @property
    def url(self) -> str: ...
    # Attributes assigned by monkey-patching in test client ClientHandler.__call__()
    wsgi_request: WSGIRequest
    # Attributes assigned by monkey-patching in test client Client.request()
    client: Client
    request: Dict[str, Any]
    templates: List[Template]
    context: Context
    resolver_match: ResolverMatch
    def json(self) -> Any: ...
    def getvalue(self) -> bytes: ...

class StreamingHttpResponse(HttpResponseBase):
    content: Any
    streaming_content: Union[Iterable[bytes], AsyncIterable[bytes]]
    def __init__(
        self,
        streaming_content: Union[Iterable[bytes], AsyncIterable[bytes]] = ...,
        *args: Any,
        **kwargs: Any
    ) -> None: ...
    def getvalue(self) -> bytes: ...

class FileResponse(StreamingHttpResponse):
    client: Client
    context: None
    file_to_stream: Optional[BytesIO]
    request: Dict[str, str]
    resolver_match: ResolverMatch
    templates: List[Any]
    wsgi_request: WSGIRequest
    block_size: int = ...
    as_attachment: bool = ...
    filename: str = ...
    def __init__(
        self, *args: Any, as_attachment: bool = ..., filename: str = ..., **kwargs: Any
    ) -> None: ...
    def set_headers(self, filelike: BytesIO) -> None: ...
    def json(self) -> Dict[str, Any]: ...

class HttpResponseRedirectBase(HttpResponse):
    allowed_schemes: List[str] = ...
    def __init__(self, redirect_to: str, *args: Any, **kwargs: Any) -> None: ...

class HttpResponseRedirect(HttpResponseRedirectBase): ...
class HttpResponsePermanentRedirect(HttpResponseRedirectBase): ...

class HttpResponseNotModified(HttpResponse):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class HttpResponseBadRequest(HttpResponse): ...
class HttpResponseNotFound(HttpResponse): ...
class HttpResponseForbidden(HttpResponse): ...

class HttpResponseNotAllowed(HttpResponse):
    def __init__(
        self, permitted_methods: Iterable[str], *args: Any, **kwargs: Any
    ) -> None: ...

class HttpResponseGone(HttpResponse): ...
class HttpResponseServerError(HttpResponse): ...
class Http404(Exception): ...

class JsonResponse(HttpResponse):
    def __init__(
        self,
        data: Any,
        encoder: Type[JSONEncoder] = ...,
        safe: bool = ...,
        json_dumps_params: Optional[Dict[str, Any]] = ...,
        **kwargs: Any
    ) -> None: ...



from typing import IO, Literal
from contextlib import closing as closing
from numpy import dtype, ndarray
from scipy.sparse import spmatrix
from .._typing import PathLike, Int, ArrayLike, MatrixLike
from ..utils import check_array as check_array, IS_PYPY as IS_PYPY
import io
import os.path

import numpy as np
import scipy.sparse as sp


def load_svmlight_file(
    f: PathLike | str | IO | int,
    *,
    n_features: None | Int = None,
    dtype: dtype = ...,
    multilabel: bool = False,
    zero_based: Literal["auto", "auto"] | bool = "auto",
    query_id: bool = False,
    offset: Int = 0,
    length: Int = ...,
) -> tuple[spmatrix, ndarray, ndarray]:
    ...


def load_svmlight_files(
    files: PathLike | ArrayLike | IO | int,
    *,
    n_features: None | Int = None,
    dtype: dtype = ...,
    multilabel: bool = False,
    zero_based: Literal["auto", "auto"] | bool = "auto",
    query_id: bool = False,
    offset: Int = 0,
    length: Int = ...,
) -> list[ndarray]:
    ...


def dump_svmlight_file(
    X: MatrixLike | ArrayLike,
    y: MatrixLike,
    f: str | IO,
    *,
    zero_based: bool = True,
    comment: None | str = None,
    query_id: None | ArrayLike = None,
    multilabel: bool = False,
):
    ...



"""
This type stub file was generated by pyright.
"""

from IPython.core.oinspect import OInfo

"""
Simple utility for splitting user input. This is used by both inputsplitter and
prefilter.

Authors:

* Brian Granger
* Fernando Perez
"""
line_split = ...
def split_user_input(line, pattern=...):
    """Split user input into initial whitespace, escape character, function part
    and the rest.
    """
    ...

class LineInfo:
    """A single line of input and associated info.

    Includes the following as properties:

    line
      The original, raw line

    continue_prompt
      Is this line a continuation in a sequence of multiline input?

    pre
      Any leading whitespace.

    esc
      The escape character(s) in pre or the empty string if there isn't one.
      Note that '!!' and '??' are possible values for esc. Otherwise it will
      always be a single character.

    ifun
      The 'function part', which is basically the maximal initial sequence
      of valid python identifiers and the '.' character. This is what is
      checked for alias and magic transformations, used for auto-calling,
      etc. In contrast to Python identifiers, it may start with "%" and contain
      "*".

    the_rest
      Everything else on the line.
    """
    def __init__(self, line, continue_prompt=...) -> None:
        ...
    
    def ofind(self, ip) -> OInfo:
        """Do a full, attribute-walking lookup of the ifun in the various
        namespaces for the given IPython InteractiveShell instance.

        Return a dict with keys: {found, obj, ospace, ismagic}

        Note: can cause state changes because of calling getattr, but should
        only be run if autocall is on and if the line hasn't matched any
        other, less dangerous handlers.

        Does cache the results of the call, so can be called multiple times
        without worrying about *further* damaging state.
        """
        ...
    
    def __str__(self) -> str:
        ...
    





from numpy.typing import ArrayLike, NDArray
from typing import Literal
from ..scene.visuals import LinePlot, Spectrogram, Image, Mesh, Polygon, Volume
from ..scene.widgets.colorbar import ColorBarWidget
from ..color import Color, Colormap
from ..geometry.meshdata import MeshData

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from .. import scene
from ..io import read_mesh
from ..geometry import MeshData

__all__ = ["PlotWidget"]

class PlotWidget(scene.Widget):
    def __init__(self, *args, **kwargs): ...
    def _configure_2d(self, fg_color=None): ...
    def _configure_3d(self): ...
    def histogram(
        self,
        data: ArrayLike,
        bins: ArrayLike | int = 10,
        color: Color | str = "w",
        orientation: Literal["h", "v"] = "h",
    ) -> Polygon: ...
    def image(
        self, data: NDArray, cmap: str = "cubehelix", clim: str | tuple = "auto", fg_color: Color | None = None, **kwargs
    ) -> Image: ...
    def mesh(
        self,
        vertices: ArrayLike | None = None,
        faces: ArrayLike | None = None,
        vertex_colors: ArrayLike | None = None,
        face_colors: ArrayLike | None = None,
        color: Color = ...,
        fname: str | None = None,
        meshdata: None | MeshData = None,
        shading: str = "auto",
    ) -> Mesh: ...
    def plot(
        self,
        data,
        color: Color | str = "k",
        symbol: str | None = None,
        line_kind: str = "-",
        width: float = 1.0,
        marker_size: float = 10.0,
        edge_color: Color | str = "k",
        face_color: Color | str = "b",
        edge_width: float = 1.0,
        title: str | None = None,
        xlabel: str | None = None,
        ylabel: str | None = None,
        connect: str | ArrayLike = "strip",
    ) -> LinePlot: ...
    def spectrogram(
        self,
        x: ArrayLike,
        n_fft: int = 256,
        step: None | int = None,
        fs: float = 1.0,
        window: str | None = "hann",
        normalize: bool = False,
        color_scale: Literal["linear", "log"] = "log",
        cmap: str = "cubehelix",
        clim: str | tuple = "auto",
    ) -> Spectrogram: ...
    def volume(
        self,
        vol: NDArray,
        clim: tuple[float, float] | None = None,
        method: Literal["mip", "iso", "translucent", "additive"] = "mip",
        threshold: float | None = None,
        cmap: str = "grays",
        **kwargs,
    ) -> Volume: ...
    def surface(self, zdata: ArrayLike, **kwargs): ...
    def colorbar(
        self,
        cmap: str | Colormap,
        position: Literal["left", "right", "top", "bottom"] = "right",
        label: str = "",
        clim: tuple[float, float] = ...,
        border_width: float = 0.0,
        border_color: str | Color = "black",
        **kwargs,
    ) -> ColorBarWidget: ...



from ...classes.graph import Graph

# Define the default maximum flow function to use for the undelying
# maximum flow computations
from ...algorithms.flow import (
    edmonds_karp,
    preflow_push,
    shortest_augmenting_path,
)
from ...exception import NetworkXNoPath

default_flow_func = ...
from itertools import filterfalse as _filterfalse

# Functions to build auxiliary data structures.
from .utils import build_auxiliary_edge_connectivity, build_auxiliary_node_connectivity

__all__ = ["edge_disjoint_paths", "node_disjoint_paths"]

def edge_disjoint_paths(
    G: Graph,
    s,
    t,
    flow_func=None,
    cutoff: int | None = None,
    auxiliary=None,
    residual=None,
): ...
def node_disjoint_paths(
    G: Graph,
    s,
    t,
    flow_func=None,
    cutoff: int | None = None,
    auxiliary=None,
    residual=None,
): ...



from numpy.typing import NDArray
from typing import Callable
import numpy as np
import scipy.ndimage as ndi

__all__ = ["blur_effect"]

def blur_effect(
    image: NDArray,
    h_size: int = 11,
    channel_axis: int | None = None,
    reduce_func: Callable = ...,
): ...



from functools import partial
import contextlib
from typing import Callable, Iterator

class MatplotlibDeprecationWarning(DeprecationWarning): ...

def warn_deprecated(
    since: str,
    *,
    message: str = ...,
    name: str = ...,
    alternative: str = ...,
    pending: bool = ...,
    obj_type: str = ...,
    addendum: str = ...,
    removal: str = ...
) -> None: ...
def deprecated(
    since: str,
    *,
    message: str = ...,
    name: str = ...,
    alternative: str = ...,
    pending: bool = ...,
    obj_type: str = ...,
    addendum: str = ...,
    removal: str = ...
) -> Callable: ...

class deprecate_privatize_attribute:
    def __init__(self, *args, **kwargs) -> None: ...
    def __set_name__(self, owner, name) -> None: ...

DECORATORS: dict = ...

def rename_parameter(
    since: str, old: str, new: str, func: None | Callable = ...
) -> partial | Callable: ...

class _deprecated_parameter_class:
    def __repr__(self): ...

def delete_parameter(
    since: str, name: str, func: None | Callable = ..., **kwargs
) -> partial | Callable: ...
def make_keyword_only(
    since: str, name: str, func: None | Callable = ...
) -> partial | Callable: ...
def deprecate_method_override(method, obj, *, allow_empty: bool = ..., **kwargs): ...
@contextlib.contextmanager
def suppress_matplotlib_deprecation_warning() -> Iterator[None]: ...



import numpy as np
from ._typing import *

CharMetrics = ...
CompositePart = ...

class AFM:
    def __init__(self, fh) -> None: ...
    def get_bbox_char(self, c: str, isord: bool = False): ...
    def string_width_height(self, s: str) -> tuple[float, float]: ...
    def get_str_bbox_and_descent(self, s: str) -> tuple[float, ...]: ...
    def get_str_bbox(self, s: str)-> tuple[float, ...]: ...
    def get_name_char(self, c: str, isord: bool = False)-> str: ...
    def get_width_char(self, c: str, isord: bool =False)-> float: ...
    def get_width_from_char_name(self, name: str)-> float: ...
    def get_height_char(self, c: str, isord: bool = False)-> float: ...
    def get_kern_dist(self, c1: str, c2: str)-> float: ...
    def get_kern_dist_from_name(self, name1: str, name2: str)-> float: ...
    def get_fontname(self) -> str: ...
    @property
    def postscript_name(self)-> str: ...
    def get_fullname(self) -> str: ...
    def get_familyname(self) -> str: ...
    @property
    def family_name(self) -> str: ...
    def get_weight(self) -> str: ...
    def get_angle(self) -> float: ...
    def get_capheight(self) -> float: ...
    def get_xheight(self) -> float: ...
    def get_underline_thickness(self) -> float: ...
    def get_horizontal_stem_width(self) -> float | None: ...
    def get_vertical_stem_width(self) -> float | None: ...



from numpy import ndarray
from numpy.typing import NDArray
from warnings import warn
import numpy as np
import scipy.ndimage as ndi
from .. import measure
from .._shared.utils import remove_arg
from .._shared.coord import ensure_spacing

def _get_high_intensity_peaks(image, mask, num_peaks, min_distance, p_norm): ...
def _get_peak_mask(image, footprint, threshold, mask=None): ...
def _exclude_border(label, border_width): ...
def _get_threshold(image, threshold_abs, threshold_rel): ...
def _get_excluded_border_width(image, min_distance, exclude_border): ...
@remove_arg("indices", changed_version="0.20")
def peak_local_max(
    image: NDArray,
    min_distance: int = 1,
    threshold_abs: float | None = None,
    threshold_rel: float | None = None,
    exclude_border=True,
    indices: bool = True,
    num_peaks: int = ...,
    footprint: NDArray | None = None,
    labels: NDArray | None = None,
    num_peaks_per_label: int = ...,
    p_norm: float = ...,
) -> np.ndarray: ...
def _prominent_peaks(
    image, min_xdistance=1, min_ydistance=1, threshold=None, num_peaks=np.inf
): ...



from typing import Sequence
from .gridspec import SubplotSpec
from .backend_bases import RendererBase
from .figure import Figure
from .axes import Axes, SubplotBase

def auto_adjust_subplotpars(
    fig,
    renderer,
    nrows_ncols: tuple[int, int],
    num1num2_list: Sequence[tuple[int, int]],
    subplot_list: Sequence[SubplotBase],
    ax_bbox_list=None,
    pad: float = 1.08,
    h_pad: float|None = None,
    w_pad: float|None = None,
    rect: tuple[float, float, float, float]|None = None,
) -> dict | None: ...
def get_renderer(fig: Figure) -> RendererBase: ...
def get_subplotspec_list(axes_list, grid_spec=None) -> list[SubplotSpec]: ...
def get_tight_layout_figure(
    fig: Figure,
    axes_list: list[Axes],
    subplotspec_list: list,
    renderer: RendererBase,
    pad: float = 1.08,
    h_pad: float|None = None,
    w_pad: float|None = None,
    rect: tuple[float, float, float, float]|None = None,
) -> SubplotSpec | None: ...



from typing import Literal, Callable
from numpy.typing import ArrayLike

import itertools
import warnings
from collections import defaultdict

from .classes.graph import Graph
from .utils import not_implemented_for

__all__ = [
    "from_numpy_matrix",
    "to_numpy_matrix",
    "from_pandas_adjacency",
    "to_pandas_adjacency",
    "from_pandas_edgelist",
    "to_pandas_edgelist",
    "to_numpy_recarray",
    "from_scipy_sparse_array",
    "from_scipy_sparse_matrix",
    "to_scipy_sparse_array",
    "to_scipy_sparse_matrix",
    "from_numpy_array",
    "to_numpy_array",
]

def to_pandas_adjacency(
    G: Graph,
    nodelist: ArrayLike | None = None,
    dtype=None,
    order=None,
    multigraph_weight=...,
    weight: str | None = "weight",
    nonedge: float = 0.0,
): ...
def from_pandas_adjacency(df, create_using=None): ...
def to_pandas_edgelist(
    G: Graph,
    source: str | int = "source",
    target: str | int = "target",
    nodelist: ArrayLike | None = None,
    dtype=None,
    order=None,
    edge_key=None,
): ...
def from_pandas_edgelist(
    df,
    source: str | int = "source",
    target: str | int = "target",
    edge_attr=None,
    create_using=None,
    edge_key=None,
): ...
def to_numpy_matrix(
    G: Graph,
    nodelist: ArrayLike | None = None,
    dtype=None,
    order: Literal["C", "F"] | None = None,
    multigraph_weight=...,
    weight="weight",
    nonedge=0.0,
): ...
def from_numpy_matrix(A, parallel_edges: bool = False, create_using=None): ...
def to_numpy_recarray(
    G: Graph,
    nodelist: ArrayLike | None = None,
    dtype=None,
    order: Literal["C", "F"] | None = None,
): ...
def to_scipy_sparse_array(
    G: Graph,
    nodelist: ArrayLike | None = None,
    dtype=None,
    weight="weight",
    format="csr",
): ...
def to_scipy_sparse_matrix(
    G: Graph,
    nodelist: ArrayLike | None = None,
    dtype=None,
    weight="weight",
    format="csr",
): ...
def from_scipy_sparse_matrix(
    A, parallel_edges: bool = False, create_using=None, edge_attribute: str = "weight"
): ...
def from_scipy_sparse_array(
    A, parallel_edges: bool = False, create_using=None, edge_attribute: str = "weight"
): ...
def to_numpy_array(
    G: Graph,
    nodelist: ArrayLike | None = None,
    dtype=None,
    order: Literal["C", "F"] | None = None,
    multigraph_weight: Callable = ...,
    weight="weight",
    nonedge=0.0,
): ...
def from_numpy_array(A, parallel_edges: bool = False, create_using=None): ...



# Helpers for current-flow betweenness and current-flow closness
# Lazy computations for inverse Laplacian and flow-matrix rows.

from ...classes.graph import Graph

def flow_matrix_row(G: Graph, weight=None, dtype=..., solver="lu"): ...

# Class to compute the inverse laplacian only for specified rows
# Allows computation of the current-flow matrix without storing entire
# inverse laplacian matrix
class InverseLaplacian:
    def __init__(self, L, width=None, dtype=None): ...
    def init_solver(self, L): ...
    def solve(self, r): ...
    def solve_inverse(self, r): ...
    def get_rows(self, r1, r2): ...
    def get_row(self, r): ...
    def width(self, L): ...

class FullInverseLaplacian(InverseLaplacian):
    def init_solver(self, L): ...
    def solve(self, rhs): ...
    def solve_inverse(self, r): ...

class SuperLUInverseLaplacian(InverseLaplacian):
    def init_solver(self, L): ...
    def solve_inverse(self, r): ...
    def solve(self, rhs): ...

class CGInverseLaplacian(InverseLaplacian):
    def init_solver(self, L): ...
    def solve(self, rhs): ...
    def solve_inverse(self, r): ...



from collections.abc import Iterable
from numpy.typing import ArrayLike
from ..classes.graph import Graph

# Original author: D. Eppstein, UC Irvine, August 12, 2003.
# The original code at https://www.ics.uci.edu/~eppstein/PADS/ is public domain.
from ..classes.graph import Graph
from ..exception import NetworkXError
from ..readwrite.graph6 import data_to_n, n_to_data
from ..utils import not_implemented_for, open_file

__all__ = ["from_sparse6_bytes", "read_sparse6", "to_sparse6_bytes", "write_sparse6"]

def from_sparse6_bytes(string: str) -> Graph: ...
def to_sparse6_bytes(
    G: Graph, nodes: ArrayLike | Iterable | None = None, header: bool = True
): ...
def read_sparse6(path): ...
def write_sparse6(
    G: Graph, path, nodes: ArrayLike | Iterable | None = None, header: bool = True
): ...



bn = ...

def set_use_bottleneck(v: bool = ...) -> None: ...

class disallow:
    dtypes = ...
    def __init__(self, *dtypes) -> None: ...
    def check(self, obj) -> bool: ...
    def __call__(self, f): ...

class bottleneck_switch:
    name = ...
    kwargs = ...
    def __init__(self, name=..., **kwargs) -> None: ...
    def __call__(self, alt): ...

def nanany(values, axis=..., skipna: bool = ..., mask=...): ...
def nanall(values, axis=..., skipna: bool = ..., mask=...): ...
def nansum(values, axis=..., skipna: bool = ..., min_count: int = ..., mask=...): ...
def nanmean(values, axis=..., skipna: bool = ..., mask=...): ...
def nanmedian(values, axis=..., skipna: bool = ..., mask=...): ...
def nanstd(values, axis=..., skipna: bool = ..., ddof: int = ..., mask=...): ...
def nanvar(values, axis=..., skipna: bool = ..., ddof: int = ..., mask=...): ...
def nansem(values, axis=..., skipna: bool = ..., ddof: int = ..., mask=...): ...

nanmin = ...
nanmax = ...

def nanargmax(values, axis=..., skipna: bool = ..., mask=...): ...
def nanargmin(values, axis=..., skipna: bool = ..., mask=...): ...
def nanskew(values, axis=..., skipna: bool = ..., mask=...): ...
def nankurt(values, axis=..., skipna: bool = ..., mask=...): ...
def nanprod(values, axis=..., skipna: bool = ..., min_count: int = ..., mask=...): ...
def nancorr(a, b, method: str = ..., min_periods=...): ...
def get_corr_func(method): ...
def nancov(a, b, min_periods=...): ...
def make_nancomp(op): ...

nangt = ...
nange = ...
nanlt = ...
nanle = ...
naneq = ...
nanne = ...

def nanpercentile(values, q, axis, na_value, mask, ndim, interpolation): ...



from typing import Any, ClassVar, Iterator, TypeVar
from itertools import chain as chain
from numpy import dtype
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from ._hashing_fast import transform as _hashing_transform
from numbers import Integral as Integral
from ..base import BaseEstimator, TransformerMixin
from scipy.sparse import spmatrix
from .._typing import Int

FeatureHasher_Self = TypeVar("FeatureHasher_Self", bound="FeatureHasher")

# Author: Lars Buitinck
# License: BSD 3 clause


import numpy as np
import scipy.sparse as sp


class FeatureHasher(TransformerMixin, BaseEstimator):

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_features: Int = ...,
        *,
        input_type: str = "dict",
        dtype: dtype = ...,
        alternate_sign: bool = True,
    ) -> None:
        ...

    def fit(
        self: FeatureHasher_Self, X: Any = None, y: Any = None
    ) -> FeatureHasher_Self:
        ...

    def transform(self, raw_X: Iterator[Any] | Iterator[Iterator]) -> spmatrix:
        ...



from typing import Any, List, Optional, Set, Tuple, Type

from django.apps.config import AppConfig
from django.db.models.base import Model

def popen_wrapper(
    args: List[str], stdout_encoding: str = ...
) -> Tuple[str, str, int]: ...
def handle_extensions(extensions: List[str]) -> Set[str]: ...
def find_command(
    cmd: str, path: Optional[str] = ..., pathext: Optional[str] = ...
) -> Optional[str]: ...
def get_random_secret_key() -> Any: ...
def parse_apps_and_model_labels(
    labels: List[str],
) -> Tuple[Set[Type[Model]], Set[AppConfig]]: ...
def get_command_line_option(argv: Any, option: Any) -> Any: ...



"""
This type stub file was generated by pyright.
"""

import pytest

def pytest_collection_modifyitems(items): # -> None:
    """This function is automatically run by pytest passing all collected test
    functions.

    We use it to add asyncio marker to all async tests and assert we don't use
    test functions that are async generators which wouldn't make sense.
    """
    ...

def get_ipython():
    ...

@pytest.fixture(scope='session', autouse=True)
def work_path():
    ...

def nopage(strng, start=..., screen_lines=..., pager_cmd=...): # -> None:
    ...

def xsys(self, cmd): # -> None:
    """Replace the default system call with a capturing one for doctest.
    """
    ...

def inject(): # -> None:
    ...




# Python: 3.8.8 (tags/v3.8.8:024d805, Feb 19 2021, 13:18:16) [MSC v.1928 64 bit (AMD64)]
# Library: matplotlib, version: 3.4.0
# Module: matplotlib.ft2font, version: unspecified
import typing
import builtins as _mod_builtins

BOLD: int
EXTERNAL_STREAM: int
FAST_GLYPHS: int
FIXED_SIZES: int
FIXED_WIDTH: int
class FT2Font(_mod_builtins.object):
    'Create a new FT2Font object.\n\nAttributes\n----------\nnum_faces\n    Number of faces in file.\nface_flags, style_flags : int\n    Face and style flags; see the ft2font constants.\nnum_glyphs\n    Number of glyphs in the face.\nfamily_name, style_name\n    Face family and style name.\nnum_fixed_sizes\n    Number of bitmap in the face.\nscalable\n    Whether face is scalable; attributes after this one are only\n    defined for scalable faces.\nbbox\n    Face global bounding box (xmin, ymin, xmax, ymax).\nunits_per_EM\n    Number of font units covered by the EM.\nascender, descender\n    Ascender and descender in 26.6 units.\nheight\n    Height in 26.6 units; used to compute a default line spacing\n    (baseline-to-baseline distance).\nmax_advance_width, max_advance_height\n    Maximum horizontal and vertical cursor advance for all glyphs.\nunderline_position, underline_thickness\n    Vertical position and thickness of the underline bar.\npostscript_name\n    PostScript name of the font.\n'
    def __init__(self, *args, **kwargs) -> None:
        'Create a new FT2Font object.\n\nAttributes\n----------\nnum_faces\n    Number of faces in file.\nface_flags, style_flags : int\n    Face and style flags; see the ft2font constants.\nnum_glyphs\n    Number of glyphs in the face.\nfamily_name, style_name\n    Face family and style name.\nnum_fixed_sizes\n    Number of bitmap in the face.\nscalable\n    Whether face is scalable; attributes after this one are only\n    defined for scalable faces.\nbbox\n    Face global bounding box (xmin, ymin, xmax, ymax).\nunits_per_EM\n    Number of font units covered by the EM.\nascender, descender\n    Ascender and descender in 26.6 units.\nheight\n    Height in 26.6 units; used to compute a default line spacing\n    (baseline-to-baseline distance).\nmax_advance_width, max_advance_height\n    Maximum horizontal and vertical cursor advance for all glyphs.\nunderline_position, underline_thickness\n    Vertical position and thickness of the underline bar.\npostscript_name\n    PostScript name of the font.\n'
        ...
    
    @classmethod
    def __init_subclass__(cls) -> None:
        'This method is called when a class is subclassed.\n\nThe default implementation does nothing. It may be\noverridden to extend subclasses.\n'
        ...
    
    @classmethod
    def __subclasshook__(cls, subclass: typing.Any) -> bool:
        'Abstract classes can override this to customize issubclass().\n\nThis is invoked early on by abc.ABCMeta.__subclasscheck__().\nIt should return True, False or NotImplemented.  If it returns\nNotImplemented, the normal algorithm is used.  Otherwise, it\noverrides the normal algorithm (and the outcome is cached).\n'
        ...
    
    @property
    def ascender(self) -> typing.Any:
        ...
    
    @property
    def bbox(self) -> typing.Any:
        ...
    
    def clear(self) -> typing.Any:
        'Clear all the glyphs, reset for a new call to `.set_text`.\n'
        ...
    
    @property
    def descender(self) -> typing.Any:
        ...
    
    def draw_glyph_to_bitmap(self, bitmap, x, y, glyph) -> typing.Any:
        'Draw a single glyph to the bitmap at pixel locations x, y\nNote it is your responsibility to set up the bitmap manually\nwith ``set_bitmap_size(w, h)`` before this call is made.\n\nIf you want automatic layout, use `.set_text` in combinations with\n`.draw_glyphs_to_bitmap`.  This function is instead intended for people\nwho want to render individual glyphs (e.g., returned by `.load_char`)\nat precise locations.\n'
        ...
    
    def draw_glyphs_to_bitmap(self) -> typing.Any:
        'Draw the glyphs that were loaded by `.set_text` to the bitmap.\nThe bitmap size will be automatically set to include the glyphs.\n'
        ...
    
    @property
    def face_flags(self) -> typing.Any:
        ...
    
    @property
    def family_name(self) -> typing.Any:
        ...
    
    @property
    def fname(self) -> typing.Any:
        ...
    
    def get_bitmap_offset(self) -> typing.Any:
        'Get the (x, y) offset in 26.6 subpixels for the bitmap if ink hangs left or below (0, 0).\nSince Matplotlib only supports left-to-right text, y is always 0.\n'
        ...
    
    def get_char_index(self, codepoint) -> typing.Any:
        'Return the glyph index corresponding to a character *codepoint*.\n'
        ...
    
    def get_charmap(self) -> typing.Any:
        'Return a dict that maps the character codes of the selected charmap\n(Unicode by default) to their corresponding glyph indices.\n'
        ...
    
    def get_descent(self) -> typing.Any:
        'Get the descent in 26.6 subpixels of the current string set by `.set_text`.\nThe rotation of the string is accounted for.  To get the descent\nin pixels, divide this value by 64.\n'
        ...
    
    def get_glyph_name(self, index) -> typing.Any:
        "Retrieve the ASCII name of a given glyph *index* in a face.\n\nDue to Matplotlib's internal design, for fonts that do not contain glyph\nnames (per FT_FACE_FLAG_GLYPH_NAMES), this returns a made-up name which\ndoes *not* roundtrip through `.get_name_index`.\n"
        ...
    
    def get_image(self) -> typing.Any:
        'Return the underlying image buffer for this font object.\n'
        ...
    
    def get_kerning(self, left, right, mode) -> typing.Any:
        'Get the kerning between *left* and *right* glyph indices.\n*mode* is a kerning mode constant:\n  KERNING_DEFAULT  - Return scaled and grid-fitted kerning distances\n  KERNING_UNFITTED - Return scaled but un-grid-fitted kerning distances\n  KERNING_UNSCALED - Return the kerning vector in original font units\n'
        ...
    
    def get_name_index(self, name) -> typing.Any:
        "Return the glyph index of a given glyph *name*.\nThe glyph index 0 means 'undefined character code'.\n"
        ...
    
    def get_num_glyphs(self) -> typing.Any:
        'Return the number of loaded glyphs.\n'
        ...
    
    def get_path(self) -> typing.Any:
        'Get the path data from the currently loaded glyph as a tuple of vertices, codes.\n'
        ...
    
    def get_ps_font_info(self) -> typing.Any:
        'Return the information in the PS Font Info structure.\n'
        ...
    
    def get_sfnt(self) -> typing.Any:
        'Load the entire SFNT names table, as a dict whose keys are\n(platform-ID, ISO-encoding-scheme, language-code, and description)\ntuples.\n'
        ...
    
    def get_sfnt_table(self, name) -> typing.Any:
        'Return one of the following SFNT tables: head, maxp, OS/2, hhea, vhea, post, or pclt.\n'
        ...
    
    def get_width_height(self) -> typing.Any:
        'Get the width and height in 26.6 subpixels of the current string set by `.set_text`.\nThe rotation of the string is accounted for.  To get width and height\nin pixels, divide these values by 64.\n'
        ...
    
    def get_xys(self) -> typing.Any:
        'Get the xy locations of the current glyphs.\n'
        ...
    
    @property
    def height(self) -> typing.Any:
        ...
    
    def load_char(self, charcode, flags) -> typing.Any:
        'Load character with *charcode* in current fontfile and set glyph.\n*flags* can be a bitwise-or of the LOAD_XXX constants;\nthe default value is LOAD_FORCE_AUTOHINT.\nReturn value is a Glyph object, with attributes\n  width          # glyph width\n  height         # glyph height\n  bbox           # the glyph bbox (xmin, ymin, xmax, ymax)\n  horiBearingX   # left side bearing in horizontal layouts\n  horiBearingY   # top side bearing in horizontal layouts\n  horiAdvance    # advance width for horizontal layout\n  vertBearingX   # left side bearing in vertical layouts\n  vertBearingY   # top side bearing in vertical layouts\n  vertAdvance    # advance height for vertical layout\n'
        ...
    
    def load_glyph(self, glyphindex, flags) -> typing.Any:
        'Load character with *glyphindex* in current fontfile and set glyph.\n*flags* can be a bitwise-or of the LOAD_XXX constants;\nthe default value is LOAD_FORCE_AUTOHINT.\nReturn value is a Glyph object, with attributes\n  width          # glyph width\n  height         # glyph height\n  bbox           # the glyph bbox (xmin, ymin, xmax, ymax)\n  horiBearingX   # left side bearing in horizontal layouts\n  horiBearingY   # top side bearing in horizontal layouts\n  horiAdvance    # advance width for horizontal layout\n  vertBearingX   # left side bearing in vertical layouts\n  vertBearingY   # top side bearing in vertical layouts\n  vertAdvance    # advance height for vertical layout\n'
        ...
    
    @property
    def max_advance_height(self) -> typing.Any:
        ...
    
    @property
    def max_advance_width(self) -> typing.Any:
        ...
    
    @property
    def num_charmaps(self) -> typing.Any:
        ...
    
    @property
    def num_faces(self) -> typing.Any:
        ...
    
    @property
    def num_fixed_sizes(self) -> typing.Any:
        ...
    
    @property
    def num_glyphs(self) -> typing.Any:
        ...
    
    @property
    def postscript_name(self) -> typing.Any:
        ...
    
    @property
    def scalable(self) -> typing.Any:
        ...
    
    def select_charmap(self, i) -> typing.Any:
        'Select a charmap by its FT_Encoding number.\n'
        ...
    
    def set_charmap(self, i) -> typing.Any:
        'Make the i-th charmap current.\n'
        ...
    
    def set_size(self, ptsize, dpi) -> typing.Any:
        'Set the point size and dpi of the text.\n'
        ...
    
    def set_text(self, string, angle, flags) -> typing.Any:
        'Set the text *string* and *angle*.\n*flags* can be a bitwise-or of the LOAD_XXX constants;\nthe default value is LOAD_FORCE_AUTOHINT.\nYou must call this before `.draw_glyphs_to_bitmap`.\nA sequence of x,y positions is returned.\n'
        ...
    
    @property
    def style_flags(self) -> typing.Any:
        ...
    
    @property
    def style_name(self) -> typing.Any:
        ...
    
    @property
    def underline_position(self) -> typing.Any:
        ...
    
    @property
    def underline_thickness(self) -> typing.Any:
        ...
    
    @property
    def units_per_EM(self) -> typing.Any:
        ...
    
    def __getattr__(self, name) -> typing.Any:
        ...
    

class FT2Image(_mod_builtins.object):
    def __init__(self, *args, **kwargs) -> None:
        ...
    
    @classmethod
    def __init_subclass__(cls) -> None:
        'This method is called when a class is subclassed.\n\nThe default implementation does nothing. It may be\noverridden to extend subclasses.\n'
        ...
    
    @classmethod
    def __subclasshook__(cls, subclass: typing.Any) -> bool:
        'Abstract classes can override this to customize issubclass().\n\nThis is invoked early on by abc.ABCMeta.__subclasscheck__().\nIt should return True, False or NotImplemented.  If it returns\nNotImplemented, the normal algorithm is used.  Otherwise, it\noverrides the normal algorithm (and the outcome is cached).\n'
        ...
    
    def draw_rect(self, x0, y0, x1, y1) -> typing.Any:
        'Draw an empty rectangle to the image.\n'
        ...
    
    def draw_rect_filled(self, x0, y0, x1, y1) -> typing.Any:
        'Draw a filled rectangle to the image.\n'
        ...
    
    def __getattr__(self, name) -> typing.Any:
        ...
    

GLYPH_NAMES: int
HORIZONTAL: int
ITALIC: int
KERNING: int
KERNING_DEFAULT: int
KERNING_UNFITTED: int
KERNING_UNSCALED: int
LOAD_CROP_BITMAP: int
LOAD_DEFAULT: int
LOAD_FORCE_AUTOHINT: int
LOAD_IGNORE_GLOBAL_ADVANCE_WIDTH: int
LOAD_IGNORE_TRANSFORM: int
LOAD_LINEAR_DESIGN: int
LOAD_MONOCHROME: int
LOAD_NO_AUTOHINT: int
LOAD_NO_BITMAP: int
LOAD_NO_HINTING: int
LOAD_NO_RECURSE: int
LOAD_NO_SCALE: int
LOAD_PEDANTIC: int
LOAD_RENDER: int
LOAD_TARGET_LCD: int
LOAD_TARGET_LCD_V: int
LOAD_TARGET_LIGHT: int
LOAD_TARGET_MONO: int
LOAD_TARGET_NORMAL: int
LOAD_VERTICAL_LAYOUT: int
MULTIPLE_MASTERS: int
SCALABLE: int
SFNT: int
VERTICAL: int
__doc__: typing.Any
__file__: str
__freetype_build_type__: str
__freetype_version__: str
__name__: str
__package__: str
def __getattr__(name) -> typing.Any:
    ...




class HatchPatternBase: ...

class HorizontalHatch(HatchPatternBase):
    def __init__(self, hatch: str, density: int) -> None: ...
    def set_vertices_and_codes(self, vertices, codes)-> None: ...

class VerticalHatch(HatchPatternBase):
    def __init__(self, hatch: str, density: int) -> None: ...
    def set_vertices_and_codes(self, vertices, codes)-> None: ...

class NorthEastHatch(HatchPatternBase):
    def __init__(self, hatch: str, density: int) -> None: ...
    def set_vertices_and_codes(self, vertices, codes)-> None: ...

class SouthEastHatch(HatchPatternBase):
    def __init__(self, hatch: str, density: int) -> None: ...
    def set_vertices_and_codes(self, vertices, codes)-> None: ...

class Shapes(HatchPatternBase):
    filled = ...
    def __init__(self, hatch: str, density: int) -> None: ...
    def set_vertices_and_codes(self, vertices, codes)-> None: ...

class Circles(Shapes):
    def __init__(self, hatch: str, density: int) -> None: ...

class SmallCircles(Circles):
    size = ...
    def __init__(self, hatch: str, density: int) -> None: ...

class LargeCircles(Circles):
    size = ...
    def __init__(self, hatch: str, density: int) -> None: ...

class SmallFilledCircles(Circles):
    size = ...
    filled = ...
    def __init__(self, hatch: str, density: int) -> None: ...

class Stars(Shapes):
    size = ...
    filled = ...
    def __init__(self, hatch: str, density: int) -> None: ...

def get_path(hatchpattern, density=...): ...



from typing import Callable, ClassVar, Mapping, TypeVar
from numpy.random import RandomState
from ._stochastic_gradient import (
    BaseSGDClassifier,
    BaseSGDRegressor,
    DEFAULT_EPSILON as DEFAULT_EPSILON,
)
from numpy import ndarray
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from numbers import Real as Real
from .._typing import Float, Int, MatrixLike, ArrayLike

PassiveAggressiveClassifier_Self = TypeVar(
    "PassiveAggressiveClassifier_Self", bound="PassiveAggressiveClassifier"
)
PassiveAggressiveRegressor_Self = TypeVar(
    "PassiveAggressiveRegressor_Self", bound="PassiveAggressiveRegressor"
)

# Authors: Rob Zinkov, Mathieu Blondel
# License: BSD 3 clause


class PassiveAggressiveClassifier(BaseSGDClassifier):
    loss_function_: Callable = ...
    t_: int = ...
    classes_: ndarray = ...
    n_iter_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    intercept_: ndarray = ...
    coef_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        C: Float = 1.0,
        fit_intercept: bool = True,
        max_iter: Int = 1000,
        tol: None | Float = 1e-3,
        early_stopping: bool = False,
        validation_fraction: Float = 0.1,
        n_iter_no_change: Int = 5,
        shuffle: bool = True,
        verbose: Int = 0,
        loss: str = "hinge",
        n_jobs: None | int = None,
        random_state: RandomState | None | Int = None,
        warm_start: bool = False,
        class_weight: None | str | Mapping[str, float] = None,
        average: int | bool = False,
    ) -> None:
        ...

    def partial_fit(
        self: PassiveAggressiveClassifier_Self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        classes: None | ArrayLike = None,
    ) -> PassiveAggressiveClassifier_Self:
        ...

    def fit(
        self: PassiveAggressiveClassifier_Self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        coef_init: None | MatrixLike = None,
        intercept_init: None | ArrayLike = None,
    ) -> PassiveAggressiveClassifier_Self:
        ...


class PassiveAggressiveRegressor(BaseSGDRegressor):
    t_: int = ...
    n_iter_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    intercept_: ndarray = ...
    coef_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        C: Float = 1.0,
        fit_intercept: bool = True,
        max_iter: Int = 1000,
        tol: None | Float = 1e-3,
        early_stopping: bool = False,
        validation_fraction: Float = 0.1,
        n_iter_no_change: Int = 5,
        shuffle: bool = True,
        verbose: Int = 0,
        loss: str = "epsilon_insensitive",
        epsilon: Float = ...,
        random_state: RandomState | None | Int = None,
        warm_start: bool = False,
        average: int | bool = False,
    ) -> None:
        ...

    def partial_fit(
        self: PassiveAggressiveRegressor_Self, X: MatrixLike | ArrayLike, y: ArrayLike
    ) -> PassiveAggressiveRegressor_Self:
        ...

    def fit(
        self: PassiveAggressiveRegressor_Self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        coef_init: None | ArrayLike = None,
        intercept_init: None | ArrayLike = None,
    ) -> PassiveAggressiveRegressor_Self:
        ...



from typing import ClassVar, Iterable, Literal, TypeVar
from ..model_selection import BaseCrossValidator
from numpy.random import RandomState
from scipy import linalg as linalg, interpolate as interpolate
from ._base import LinearModel, LinearRegression as LinearRegression
from ..exceptions import ConvergenceWarning as ConvergenceWarning
from ..utils._param_validation import (
    Hidden as Hidden,
    Interval as Interval,
    StrOptions as StrOptions,
)
from numpy import ndarray
from numbers import Integral as Integral, Real as Real
from scipy.linalg.lapack import get_lapack_funcs as get_lapack_funcs
from ..model_selection._split import BaseShuffleSplit
from ..utils.parallel import delayed as delayed, Parallel as Parallel
from math import log as log
from ..base import RegressorMixin, MultiOutputMixin
from ..model_selection import check_cv as check_cv
from .._typing import MatrixLike, ArrayLike, Int, Float
from ..utils import (
    arrayfuncs as arrayfuncs,
    as_float_array as as_float_array,
    check_random_state as check_random_state,
)

LassoLarsIC_Self = TypeVar("LassoLarsIC_Self", bound="LassoLarsIC")
Lars_Self = TypeVar("Lars_Self", bound="Lars")
LarsCV_Self = TypeVar("LarsCV_Self", bound="LarsCV")

import sys
import warnings
import numpy as np

SOLVE_TRIANGULAR_ARGS: dict = ...


def lars_path(
    X: None | MatrixLike,
    y: None | ArrayLike,
    Xy: None | MatrixLike | ArrayLike = None,
    *,
    Gram: None | MatrixLike | str = None,
    max_iter: Int = 500,
    alpha_min: Float = 0,
    method: Literal["lar", "lasso", "lar"] = "lar",
    copy_X: bool = True,
    eps: Float = ...,
    copy_Gram: bool = True,
    verbose: Int = 0,
    return_path: bool = True,
    return_n_iter: bool = False,
    positive: bool = False,
) -> tuple[ndarray, ndarray, ndarray, int]:
    ...


def lars_path_gram(
    Xy: MatrixLike | ArrayLike,
    Gram: MatrixLike,
    *,
    n_samples: float | int,
    max_iter: Int = 500,
    alpha_min: Float = 0,
    method: Literal["lar", "lasso", "lar"] = "lar",
    copy_X: bool = True,
    eps: Float = ...,
    copy_Gram: bool = True,
    verbose: Int = 0,
    return_path: bool = True,
    return_n_iter: bool = False,
    positive: bool = False,
) -> tuple[ndarray, ndarray, ndarray, int]:
    ...


###############################################################################
# Estimator classes


class Lars(MultiOutputMixin, RegressorMixin, LinearModel):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_iter_: ArrayLike | int = ...
    intercept_: float | ArrayLike = ...
    coef_: ArrayLike = ...
    coef_path_: ArrayLike | list[ArrayLike] = ...
    active_: list[list] | list = ...
    alphas_: ArrayLike | list[ArrayLike] = ...

    _parameter_constraints: ClassVar[dict] = ...

    method: ClassVar[str] = ...
    positive: ClassVar[bool] = ...

    def __init__(
        self,
        *,
        fit_intercept: bool = True,
        verbose: int | bool = False,
        normalize: str | bool = "deprecated",
        precompute: Literal["auto", "auto"] | ArrayLike | bool = "auto",
        n_nonzero_coefs: Int = 500,
        eps: Float = ...,
        copy_X: bool = True,
        fit_path: bool = True,
        jitter: None | Float = None,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...

    def fit(
        self: Lars_Self,
        X: MatrixLike,
        y: MatrixLike | ArrayLike,
        Xy: None | MatrixLike | ArrayLike = None,
    ) -> Lars_Self:
        ...


class LassoLars(Lars):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_iter_: ArrayLike | int = ...
    intercept_: float | ArrayLike = ...
    coef_: ArrayLike = ...
    coef_path_: ArrayLike | list[ArrayLike] = ...
    active_: list[list] | list = ...
    alphas_: ArrayLike | list[ArrayLike] = ...

    _parameter_constraints: ClassVar[dict] = ...

    method: ClassVar[str] = ...

    def __init__(
        self,
        alpha: Float = 1.0,
        *,
        fit_intercept: bool = True,
        verbose: int | bool = False,
        normalize: str | bool = "deprecated",
        precompute: Literal["auto", "auto"] | ArrayLike | bool = "auto",
        max_iter: Int = 500,
        eps: Float = ...,
        copy_X: bool = True,
        fit_path: bool = True,
        positive: bool = False,
        jitter: None | Float = None,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...


class LarsCV(Lars):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_iter_: ArrayLike | int = ...
    mse_path_: ArrayLike = ...
    cv_alphas_: ArrayLike = ...
    alphas_: ArrayLike = ...
    alpha_: float = ...
    coef_path_: ArrayLike = ...
    intercept_: float = ...
    coef_: ArrayLike = ...
    active_: list[list] | list = ...

    _parameter_constraints: ClassVar[dict] = ...

    for parameter in ["n_nonzero_coefs", "jitter", "fit_path", "random_state"]:
        pass

    method: ClassVar[str] = ...

    def __init__(
        self,
        *,
        fit_intercept: bool = True,
        verbose: int | bool = False,
        max_iter: Int = 500,
        normalize: str | bool = "deprecated",
        precompute: Literal["auto", "auto"] | ArrayLike | bool = "auto",
        cv: int | BaseCrossValidator | Iterable | None | BaseShuffleSplit = None,
        max_n_alphas: Int = 1000,
        n_jobs: None | int = None,
        eps: Float = ...,
        copy_X: bool = True,
    ) -> None:
        ...

    def fit(self: LarsCV_Self, X: MatrixLike, y: ArrayLike) -> LarsCV_Self:
        ...


class LassoLarsCV(LarsCV):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    active_: list[int] = ...
    n_iter_: ArrayLike | int = ...
    mse_path_: ArrayLike = ...
    cv_alphas_: ArrayLike = ...
    alphas_: ArrayLike = ...
    alpha_: float = ...
    coef_path_: ArrayLike = ...
    intercept_: float = ...
    coef_: ArrayLike = ...

    _parameter_constraints: ClassVar[dict] = ...

    method: ClassVar[str] = ...

    def __init__(
        self,
        *,
        fit_intercept: bool = True,
        verbose: int | bool = False,
        max_iter: Int = 500,
        normalize: str | bool = "deprecated",
        precompute: Literal["auto", "auto"] | bool = "auto",
        cv: int | BaseCrossValidator | Iterable | None | BaseShuffleSplit = None,
        max_n_alphas: Int = 1000,
        n_jobs: None | int = None,
        eps: Float = ...,
        copy_X: bool = True,
        positive: bool = False,
    ) -> None:
        ...


class LassoLarsIC(LassoLars):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    noise_variance_: float = ...
    criterion_: ArrayLike = ...
    n_iter_: int = ...
    alphas_: ArrayLike | list[ArrayLike] = ...
    alpha_: float = ...
    intercept_: float = ...
    coef_: ArrayLike = ...

    _parameter_constraints: ClassVar[dict] = ...

    for parameter in ["jitter", "fit_path", "alpha", "random_state"]:
        pass

    def __init__(
        self,
        criterion: Literal["aic", "bic", "aic"] = "aic",
        *,
        fit_intercept: bool = True,
        verbose: int | bool = False,
        normalize: str | bool = "deprecated",
        precompute: Literal["auto", "auto"] | ArrayLike | bool = "auto",
        max_iter: Int = 500,
        eps: Float = ...,
        copy_X: bool = True,
        positive: bool = False,
        noise_variance: None | Float = None,
    ) -> None:
        ...

    def fit(
        self: LassoLarsIC_Self, X: MatrixLike, y: ArrayLike, copy_X: None | bool = None
    ) -> LassoLarsIC_Self:
        ...



def get_mgr_concatenation_plan(mgr, indexers): ...

class JoinUnit:
    block = ...
    indexers = ...
    shape = ...
    def __init__(self, block, shape, indexers=...) -> None: ...
    def needs_filling(self): ...
    def dtype(self): ...
    def is_na(self): ...
    def get_reindexed_values(self, empty_dtype, upcasted_na): ...

def concatenate_join_units(join_units, concat_axis, copy): ...
def is_uniform_join_units(join_units): ...
def combine_concat_plans(plans, concat_axis): ...



from typing import Literal
from matplotlib._typing import *
from matplotlib.axes import Axes

def tripcolor(
    ax: Axes,
    *args,
    alpha=...,
    norm=...,
    cmap=...,
    vmin=...,
    vmax=...,
    shading: Literal["flat", "gouraud"] = "flat",
    facecolors: ArrayLike = ...,
    **kwargs
): ...



from vispy.util.svg.shapes import Rect
import numpy as np

class Rect(object):
    def __init__(self, *args, **kwargs): ...
    @property
    def pos(self): ...
    @pos.setter
    def pos(self, p): ...
    @property
    def size(self): ...
    @size.setter
    def size(self, s): ...
    @property
    def width(self): ...
    @width.setter
    def width(self, w): ...
    @property
    def height(self): ...
    @height.setter
    def height(self, h): ...
    @property
    def left(self): ...
    @left.setter
    def left(self, x): ...
    @property
    def right(self): ...
    @right.setter
    def right(self, x): ...
    @property
    def bottom(self): ...
    @bottom.setter
    def bottom(self, y): ...
    @property
    def top(self): ...
    @top.setter
    def top(self, y): ...
    @property
    def center(self): ...
    @center.setter
    def center(self, value): ...
    def padded(self, padding: float) -> Rect: ...
    def normalized(self): ...
    def flipped(self, x: bool = False, y: bool = True) -> Rect: ...
    def __eq__(self, r): ...
    def __add__(self, a): ...
    def contains(self, x: float, y: float) -> bool: ...
    def __repr__(self): ...
    def _transform_in(self): ...
    def _transform_out(self, coords): ...



"""
This type stub file was generated by pyright.
"""

import numpy as np
from pathlib import Path
from typing import Any, Callable, Dict, List, Optional, Tuple, Union
from .basic import Booster, Dataset

"""Library with training routines of LightGBM."""
_LGBM_CustomObjectiveFunction = Callable[[Union[List, np.ndarray], Dataset], Tuple[Union[List, np.ndarray], Union[List, np.ndarray]]]
_LGBM_CustomMetricFunction = Callable[[Union[List, np.ndarray], Dataset], Tuple[str, float, bool]]
def train(params: Dict[str, Any], train_set: Dataset, num_boost_round: int = ..., valid_sets: Optional[List[Dataset]] = ..., valid_names: Optional[List[str]] = ..., fobj: Optional[_LGBM_CustomObjectiveFunction] = ..., feval: Optional[Union[_LGBM_CustomMetricFunction, List[_LGBM_CustomMetricFunction]]] = ..., init_model: Optional[Union[str, Path, Booster]] = ..., feature_name: Union[List[str], str] = ..., categorical_feature: Union[List[str], List[int], str] = ..., early_stopping_rounds: Optional[int] = ..., evals_result: Optional[Dict[str, Any]] = ..., verbose_eval: Union[bool, int, str] = ..., learning_rates: Optional[Union[List[float], Callable[[int], float]]] = ..., keep_training_booster: bool = ..., callbacks: Optional[List[Callable]] = ...) -> Booster:
    """Perform the training with given parameters.

    Parameters
    ----------
    params : dict
        Parameters for training.
    train_set : Dataset
        Data to be trained on.
    num_boost_round : int, optional (default=100)
        Number of boosting iterations.
    valid_sets : list of Dataset, or None, optional (default=None)
        List of data to be evaluated on during training.
    valid_names : list of str, or None, optional (default=None)
        Names of ``valid_sets``.
    fobj : callable or None, optional (default=None)
        Customized objective function.
        Should accept two parameters: preds, train_data,
        and return (grad, hess).

            preds : list or numpy 1-D array
                The predicted values.
                Predicted values are returned before any transformation,
                e.g. they are raw margin instead of probability of positive class for binary task.
            train_data : Dataset
                The training dataset.
            grad : list or numpy 1-D array
                The value of the first order derivative (gradient) of the loss
                with respect to the elements of preds for each sample point.
            hess : list or numpy 1-D array
                The value of the second order derivative (Hessian) of the loss
                with respect to the elements of preds for each sample point.

        For multi-class task, the preds is group by class_id first, then group by row_id.
        If you want to get i-th row preds in j-th class, the access way is score[j * num_data + i]
        and you should group grad and hess in this way as well.

    feval : callable, list of callable, or None, optional (default=None)
        Customized evaluation function.
        Each evaluation function should accept two parameters: preds, train_data,
        and return (eval_name, eval_result, is_higher_better) or list of such tuples.

            preds : list or numpy 1-D array
                The predicted values.
                If ``fobj`` is specified, predicted values are returned before any transformation,
                e.g. they are raw margin instead of probability of positive class for binary task in this case.
            train_data : Dataset
                The training dataset.
            eval_name : str
                The name of evaluation function (without whitespaces).
            eval_result : float
                The eval result.
            is_higher_better : bool
                Is eval result higher better, e.g. AUC is ``is_higher_better``.

        For multi-class task, the preds is group by class_id first, then group by row_id.
        If you want to get i-th row preds in j-th class, the access way is preds[j * num_data + i].
        To ignore the default metric corresponding to the used objective,
        set the ``metric`` parameter to the string ``"None"`` in ``params``.
    init_model : str, pathlib.Path, Booster or None, optional (default=None)
        Filename of LightGBM model or Booster instance used for continue training.
    feature_name : list of str, or 'auto', optional (default="auto")
        Feature names.
        If 'auto' and data is pandas DataFrame, data columns names are used.
    categorical_feature : list of str or int, or 'auto', optional (default="auto")
        Categorical features.
        If list of int, interpreted as indices.
        If list of str, interpreted as feature names (need to specify ``feature_name`` as well).
        If 'auto' and data is pandas DataFrame, pandas unordered categorical columns are used.
        All values in categorical features should be less than int32 max value (2147483647).
        Large values could be memory consuming. Consider using consecutive integers starting from zero.
        All negative values in categorical features will be treated as missing values.
        The output cannot be monotonically constrained with respect to a categorical feature.
    early_stopping_rounds : int or None, optional (default=None)
        Activates early stopping. The model will train until the validation score stops improving.
        Validation score needs to improve at least every ``early_stopping_rounds`` round(s)
        to continue training.
        Requires at least one validation data and one metric.
        If there's more than one, will check all of them. But the training data is ignored anyway.
        To check only the first metric, set the ``first_metric_only`` parameter to ``True`` in ``params``.
        The index of iteration that has the best performance will be saved in the ``best_iteration`` field
        if early stopping logic is enabled by setting ``early_stopping_rounds``.
    evals_result : dict or None, optional (default=None)
        Dictionary used to store all evaluation results of all the items in ``valid_sets``.
        This should be initialized outside of your call to ``train()`` and should be empty.
        Any initial contents of the dictionary will be deleted.

        .. rubric:: Example

        With a ``valid_sets`` = [valid_set, train_set],
        ``valid_names`` = ['eval', 'train']
        and a ``params`` = {'metric': 'logloss'}
        returns {'train': {'logloss': ['0.48253', '0.35953', ...]},
        'eval': {'logloss': ['0.480385', '0.357756', ...]}}.

    verbose_eval : bool or int, optional (default=True)
        Requires at least one validation data.
        If True, the eval metric on the valid set is printed at each boosting stage.
        If int, the eval metric on the valid set is printed at every ``verbose_eval`` boosting stage.
        The last boosting stage or the boosting stage found by using ``early_stopping_rounds`` is also printed.

        .. rubric:: Example

        With ``verbose_eval`` = 4 and at least one item in ``valid_sets``,
        an evaluation metric is printed every 4 (instead of 1) boosting stages.

    learning_rates : list, callable or None, optional (default=None)
        List of learning rates for each boosting round
        or a callable that calculates ``learning_rate``
        in terms of current number of round (e.g. yields learning rate decay).
    keep_training_booster : bool, optional (default=False)
        Whether the returned Booster will be used to keep training.
        If False, the returned value will be converted into _InnerPredictor before returning.
        This means you won't be able to use ``eval``, ``eval_train`` or ``eval_valid`` methods of the returned Booster.
        When your model is very large and cause the memory error,
        you can try to set this param to ``True`` to avoid the model conversion performed during the internal call of ``model_to_string``.
        You can still use _InnerPredictor as ``init_model`` for future continue training.
    callbacks : list of callable, or None, optional (default=None)
        List of callback functions that are applied at each iteration.
        See Callbacks in Python API for more information.

    Returns
    -------
    booster : Booster
        The trained Booster model.
    """
    ...

class CVBooster:
    """CVBooster in LightGBM.

    Auxiliary data structure to hold and redirect all boosters of ``cv`` function.
    This class has the same methods as Booster class.
    All method calls are actually performed for underlying Boosters and then all returned results are returned in a list.

    Attributes
    ----------
    boosters : list of Booster
        The list of underlying fitted models.
    best_iteration : int
        The best iteration of fitted model.
    """
    def __init__(self) -> None:
        """Initialize the CVBooster.

        Generally, no need to instantiate manually.
        """
        ...
    
    def __getattr__(self, name): # -> (*args: Unknown, **kwargs: Unknown) -> list[Unknown]:
        """Redirect methods call of CVBooster."""
        ...
    


def cv(params, train_set, num_boost_round=..., folds=..., nfold=..., stratified=..., shuffle=..., metrics=..., fobj=..., feval=..., init_model=..., feature_name=..., categorical_feature=..., early_stopping_rounds=..., fpreproc=..., verbose_eval=..., show_stdv=..., seed=..., callbacks=..., eval_train_metric=..., return_cvbooster=...):
    """Perform the cross-validation with given parameters.

    Parameters
    ----------
    params : dict
        Parameters for Booster.
    train_set : Dataset
        Data to be trained on.
    num_boost_round : int, optional (default=100)
        Number of boosting iterations.
    folds : generator or iterator of (train_idx, test_idx) tuples, scikit-learn splitter object or None, optional (default=None)
        If generator or iterator, it should yield the train and test indices for each fold.
        If object, it should be one of the scikit-learn splitter classes
        (https://scikit-learn.org/stable/modules/classes.html#splitter-classes)
        and have ``split`` method.
        This argument has highest priority over other data split arguments.
    nfold : int, optional (default=5)
        Number of folds in CV.
    stratified : bool, optional (default=True)
        Whether to perform stratified sampling.
    shuffle : bool, optional (default=True)
        Whether to shuffle before splitting data.
    metrics : str, list of str, or None, optional (default=None)
        Evaluation metrics to be monitored while CV.
        If not None, the metric in ``params`` will be overridden.
    fobj : callable or None, optional (default=None)
        Customized objective function.
        Should accept two parameters: preds, train_data,
        and return (grad, hess).

            preds : list or numpy 1-D array
                The predicted values.
                Predicted values are returned before any transformation,
                e.g. they are raw margin instead of probability of positive class for binary task.
            train_data : Dataset
                The training dataset.
            grad : list or numpy 1-D array
                The value of the first order derivative (gradient) of the loss
                with respect to the elements of preds for each sample point.
            hess : list or numpy 1-D array
                The value of the second order derivative (Hessian) of the loss
                with respect to the elements of preds for each sample point.

        For multi-class task, the preds is group by class_id first, then group by row_id.
        If you want to get i-th row preds in j-th class, the access way is score[j * num_data + i]
        and you should group grad and hess in this way as well.

    feval : callable, list of callable, or None, optional (default=None)
        Customized evaluation function.
        Each evaluation function should accept two parameters: preds, train_data,
        and return (eval_name, eval_result, is_higher_better) or list of such tuples.

            preds : list or numpy 1-D array
                The predicted values.
                If ``fobj`` is specified, predicted values are returned before any transformation,
                e.g. they are raw margin instead of probability of positive class for binary task in this case.
            train_data : Dataset
                The training dataset.
            eval_name : str
                The name of evaluation function (without whitespace).
            eval_result : float
                The eval result.
            is_higher_better : bool
                Is eval result higher better, e.g. AUC is ``is_higher_better``.

        For multi-class task, the preds is group by class_id first, then group by row_id.
        If you want to get i-th row preds in j-th class, the access way is preds[j * num_data + i].
        To ignore the default metric corresponding to the used objective,
        set ``metrics`` to the string ``"None"``.
    init_model : str, pathlib.Path, Booster or None, optional (default=None)
        Filename of LightGBM model or Booster instance used for continue training.
    feature_name : list of str, or 'auto', optional (default="auto")
        Feature names.
        If 'auto' and data is pandas DataFrame, data columns names are used.
    categorical_feature : list of str or int, or 'auto', optional (default="auto")
        Categorical features.
        If list of int, interpreted as indices.
        If list of str, interpreted as feature names (need to specify ``feature_name`` as well).
        If 'auto' and data is pandas DataFrame, pandas unordered categorical columns are used.
        All values in categorical features should be less than int32 max value (2147483647).
        Large values could be memory consuming. Consider using consecutive integers starting from zero.
        All negative values in categorical features will be treated as missing values.
        The output cannot be monotonically constrained with respect to a categorical feature.
    early_stopping_rounds : int or None, optional (default=None)
        Activates early stopping.
        CV score needs to improve at least every ``early_stopping_rounds`` round(s)
        to continue.
        Requires at least one metric. If there's more than one, will check all of them.
        To check only the first metric, set the ``first_metric_only`` parameter to ``True`` in ``params``.
        Last entry in evaluation history is the one from the best iteration.
    fpreproc : callable or None, optional (default=None)
        Preprocessing function that takes (dtrain, dtest, params)
        and returns transformed versions of those.
    verbose_eval : bool, int, or None, optional (default=None)
        Whether to display the progress.
        If True, progress will be displayed at every boosting stage.
        If int, progress will be displayed at every given ``verbose_eval`` boosting stage.
    show_stdv : bool, optional (default=True)
        Whether to display the standard deviation in progress.
        Results are not affected by this parameter, and always contain std.
    seed : int, optional (default=0)
        Seed used to generate the folds (passed to numpy.random.seed).
    callbacks : list of callable, or None, optional (default=None)
        List of callback functions that are applied at each iteration.
        See Callbacks in Python API for more information.
    eval_train_metric : bool, optional (default=False)
        Whether to display the train metric in progress.
        The score of the metric is calculated again after each training step, so there is some impact on performance.
    return_cvbooster : bool, optional (default=False)
        Whether to return Booster models trained on each fold through ``CVBooster``.

    Returns
    -------
    eval_hist : dict
        Evaluation history.
        The dictionary has the following format:
        {'metric1-mean': [values], 'metric1-stdv': [values],
        'metric2-mean': [values], 'metric2-stdv': [values],
        ...}.
        If ``return_cvbooster=True``, also returns trained boosters via ``cvbooster`` key.
    """
    ...




from typing import List

from pandas.core.arrays import ExtensionArray as ExtensionArray
from pandas.core.base import PandasObject as PandasObject

class Block(PandasObject):
    is_numeric: bool = ...
    is_float: bool = ...
    is_integer: bool = ...
    is_complex: bool = ...
    is_datetime: bool = ...
    is_datetimetz: bool = ...
    is_timedelta: bool = ...
    is_bool: bool = ...
    is_object: bool = ...
    is_categorical: bool = ...
    is_extension: bool = ...
    ndim = ...
    values = ...
    def __init__(self, values, placement, ndim=...) -> None: ...
    @property
    def is_view(self): ...
    @property
    def is_datelike(self): ...
    def is_categorical_astype(self, dtype): ...
    def external_values(self, dtype=...): ...
    def internal_values(self, dtype=...): ...
    def array_values(self) -> ExtensionArray: ...
    def get_values(self, dtype=...): ...
    def get_block_values(self, dtype=...): ...
    def to_dense(self): ...
    @property
    def fill_value(self): ...
    @property
    def mgr_locs(self): ...
    @mgr_locs.setter
    def mgr_locs(self, new_mgr_locs) -> None: ...
    @property
    def array_dtype(self): ...
    def make_block(self, values, placement=...) -> Block: ...
    def make_block_same_class(self, values, placement=..., ndim=...): ...
    def __len__(self) -> int: ...
    def getitem_block(self, slicer, new_mgr_locs=...): ...
    @property
    def shape(self): ...
    @property
    def dtype(self): ...
    @property
    def ftype(self): ...
    def merge(self, other): ...
    def concat_same_type(self, to_concat, placement=...): ...
    def iget(self, i): ...
    def set(self, locs, values) -> None: ...
    def delete(self, loc) -> None: ...
    def apply(self, func, **kwargs): ...
    def fillna(self, value, limit=..., inplace: bool = ..., downcast=...): ...
    def split_and_operate(self, mask, f, inplace: bool): ...
    def downcast(self, dtypes=...): ...
    def astype(self, dtype, copy: bool = ..., errors: str = ...): ...
    def convert(
        self,
        copy: bool = ...,
        datetime: bool = ...,
        numeric: bool = ...,
        timedelta: bool = ...,
        coerce: bool = ...,
    ): ...
    def to_native_types(self, slicer=..., na_rep: str = ..., quoting=..., **kwargs): ...
    def copy(self, deep: bool = ...): ...
    def replace(
        self,
        to_replace,
        value,
        inplace: bool = ...,
        filter=...,
        regex: bool = ...,
        convert: bool = ...,
    ): ...
    def setitem(self, indexer, value): ...
    def putmask(
        self,
        mask,
        new,
        align: bool = ...,
        inplace: bool = ...,
        axis: int = ...,
        transpose: bool = ...,
    ): ...
    def coerce_to_target_dtype(self, other): ...
    def interpolate(
        self,
        *,
        method: str = ...,
        axis: int = ...,
        index=...,
        inplace: bool = ...,
        limit=...,
        limit_direction: str = ...,
        limit_area=...,
        fill_value=...,
        downcast=...,
        **kwargs,
    ): ...
    def take_nd(self, indexer, axis, new_mgr_locs=..., fill_tuple=...): ...
    def diff(self, n: int, axis: int = ...) -> List[Block]: ...
    def shift(self, periods, axis: int = ..., fill_value=...): ...
    def where(
        self, other, cond, align=..., errors=..., try_cast: bool = ..., axis: int = ...
    ) -> List[Block]: ...
    def equals(self, other) -> bool: ...
    def quantile(self, qs, interpolation: str = ..., axis: int = ...): ...

class NonConsolidatableMixIn:
    def __init__(self, values, placement, ndim=...) -> None: ...
    @property
    def shape(self): ...
    def iget(self, i): ...
    def should_store(self, value): ...
    values = ...
    def set(self, locs, values, check: bool = ...) -> None: ...
    def putmask(
        self,
        mask,
        new,
        align: bool = ...,
        inplace: bool = ...,
        axis: int = ...,
        transpose: bool = ...,
    ): ...

class ExtensionBlock(NonConsolidatableMixIn, Block):
    is_extension: bool = ...
    def __init__(self, values, placement, ndim=...) -> None: ...
    @property
    def fill_value(self): ...
    @property
    def is_view(self): ...
    @property
    def is_numeric(self): ...
    def setitem(self, indexer, value): ...
    def get_values(self, dtype=...): ...
    def array_values(self) -> ExtensionArray: ...
    def to_dense(self): ...
    def to_native_types(self, slicer=..., na_rep: str = ..., quoting=..., **kwargs): ...
    def take_nd(self, indexer, axis: int = ..., new_mgr_locs=..., fill_tuple=...): ...
    def concat_same_type(self, to_concat, placement=...): ...
    def fillna(self, value, limit=..., inplace: bool = ..., downcast=...): ...
    def interpolate(
        self,
        *,
        method: str = ...,
        axis: int = ...,
        inplace: bool = ...,
        limit=...,
        fill_value=...,
        **kwargs,
    ): ...
    def diff(self, n: int, axis: int = ...) -> List[Block]: ...
    def shift(
        self, periods: int, axis: int = ..., fill_value=...
    ) -> List[ExtensionBlock]: ...
    def where(
        self, other, cond, align=..., errors=..., try_cast: bool = ..., axis: int = ...
    ) -> List[Block]: ...

class ObjectValuesExtensionBlock(ExtensionBlock):
    def external_values(self, dtype=...): ...

class NumericBlock(Block):
    is_numeric: bool = ...

class DatetimeLikeBlockMixin:
    @property
    def fill_value(self): ...
    def get_values(self, dtype=...): ...
    def iget(self, i): ...
    def shift(self, periods, axis: int = ..., fill_value=...): ...

class DatetimeBlock(DatetimeLikeBlockMixin, Block):
    is_datetime: bool = ...
    def __init__(self, values, placement, ndim=...) -> None: ...
    def astype(self, dtype, copy: bool = ..., errors: str = ...): ...
    def to_native_types(
        self, slicer=..., na_rep=..., date_format=..., quoting=..., **kwargs
    ): ...
    def should_store(self, value): ...
    def set(self, locs, values) -> None: ...
    def external_values(self): ...
    def array_values(self) -> ExtensionArray: ...

class DatetimeTZBlock(DatetimeBlock):
    is_datetimetz: bool = ...
    is_extension: bool = ...
    fill_value = ...
    @property
    def is_view(self): ...
    def get_values(self, dtype=...): ...
    def to_dense(self): ...
    def diff(self, n: int, axis: int = ...) -> List[Block]: ...
    def concat_same_type(self, to_concat, placement=...): ...
    def fillna(self, value, limit=..., inplace: bool = ..., downcast=...): ...
    def setitem(self, indexer, value): ...
    def equals(self, other) -> bool: ...
    def quantile(self, qs, interpolation: str = ..., axis: int = ...): ...

class BoolBlock(NumericBlock):
    is_bool: bool = ...
    def should_store(self, value): ...
    def replace(
        self,
        to_replace,
        value,
        inplace: bool = ...,
        filter=...,
        regex: bool = ...,
        convert: bool = ...,
    ): ...

class ObjectBlock(Block):
    is_object: bool = ...
    def __init__(self, values, placement=..., ndim: int = ...) -> None: ...
    @property
    def is_bool(self): ...
    def convert(
        self,
        copy: bool = ...,
        datetime: bool = ...,
        numeric: bool = ...,
        timedelta: bool = ...,
        coerce: bool = ...,
    ): ...
    def should_store(self, value): ...
    def replace(
        self,
        to_replace,
        value,
        inplace: bool = ...,
        filter=...,
        regex: bool = ...,
        convert: bool = ...,
    ): ...

class CategoricalBlock(ExtensionBlock):
    is_categorical: bool = ...
    def __init__(self, values, placement, ndim=...) -> None: ...
    @property
    def array_dtype(self): ...
    def to_dense(self): ...
    def to_native_types(self, slicer=..., na_rep: str = ..., quoting=..., **kwargs): ...
    def concat_same_type(self, to_concat, placement=...): ...
    def replace(
        self,
        to_replace,
        value,
        inplace: bool = ...,
        filter=...,
        regex: bool = ...,
        convert: bool = ...,
    ): ...

def get_block_type(values, dtype=...): ...
def make_block(values, placement, klass=..., ndim=..., dtype=...): ...



from typing import Any

from heapq import heappop, heappush
from itertools import count

from ..classes.graph import Graph

__all__ = ["MinHeap", "PairingHeap", "BinaryHeap"]

class MinHeap:
    ...

    def __init__(self): ...
    def min(self) -> tuple: ...
    def pop(self) -> tuple: ...
    def get(self, key, default: Any = None): ...
    def insert(self, key, value, allow_increase: bool = False) -> bool: ...
    def __nonzero__(self): ...
    def __bool__(self): ...
    def __len__(self): ...
    def __contains__(self, key): ...

class PairingHeap(MinHeap):
    ...

    def __init__(self): ...
    def min(self) -> tuple: ...
    def pop(self) -> tuple: ...
    def get(self, key, default: Any = None): ...
    def insert(self, key, value, allow_increase: bool = False) -> bool: ...
    def _link(self, root, other): ...
    def _merge_children(self, root): ...
    def _cut(self, node): ...

class BinaryHeap(MinHeap):
    def __init__(self): ...
    def min(self) -> tuple: ...
    def pop(self) -> tuple: ...
    def get(self, key, default: Any = None): ...
    def insert(self, key, value, allow_increase: bool = False) -> bool: ...



# Stubs for django.db.migrations (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from .migration import Migration as Migration
from .migration import swappable_dependency as swappable_dependency

# FIXME: Add all operations here
from .operations import *  # noqa:F403



from typing import Callable, ClassVar, Literal, Mapping, Sequence, TypeVar
from numpy.random import RandomState
from ...utils.validation import (
    check_is_fitted as check_is_fitted,
    check_consistent_length as check_consistent_length,
)
from ...utils import (
    check_random_state as check_random_state,
    resample as resample,
    compute_sample_weight as compute_sample_weight,
)
from ...utils._param_validation import Interval as Interval, StrOptions as StrOptions
from ..._loss.loss import (
    _LOSSES,
    BaseLoss as BaseLoss,
    HalfBinomialLoss as HalfBinomialLoss,
    HalfMultinomialLoss as HalfMultinomialLoss,
    HalfPoissonLoss as HalfPoissonLoss,
    PinballLoss as PinballLoss,
)
from ...metrics import check_scoring as check_scoring
from timeit import default_timer as time
from .grower import TreeGrower as TreeGrower
from abc import ABC, abstractmethod
from numpy import ndarray
from ...utils.multiclass import (
    check_classification_targets as check_classification_targets,
)
from numbers import Real as Real, Integral as Integral
from ...model_selection import train_test_split as train_test_split
from functools import partial as partial
from ...preprocessing import LabelEncoder as LabelEncoder
from ...base import (
    BaseEstimator,
    RegressorMixin,
    ClassifierMixin,
    is_classifier as is_classifier,
)
from .common import Y_DTYPE as Y_DTYPE, X_DTYPE as X_DTYPE, G_H_DTYPE as G_H_DTYPE
from ..._typing import MatrixLike, ArrayLike, Float, Int

BaseHistGradientBoosting_Self = TypeVar(
    "BaseHistGradientBoosting_Self", bound="BaseHistGradientBoosting"
)

import itertools
import warnings

import numpy as np


_LOSSES = ...


class BaseHistGradientBoosting(BaseEstimator, ABC):

    _parameter_constraints: ClassVar[dict] = ...

    @abstractmethod
    def __init__(
        self,
        loss: str,
        *,
        learning_rate,
        max_iter,
        max_leaf_nodes,
        max_depth,
        min_samples_leaf,
        l2_regularization,
        max_bins,
        categorical_features,
        monotonic_cst,
        interaction_cst,
        warm_start,
        early_stopping,
        scoring,
        validation_fraction,
        n_iter_no_change,
        tol,
        verbose,
        random_state,
    ) -> None:
        ...

    def fit(
        self: BaseHistGradientBoosting_Self,
        X: MatrixLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> BaseHistGradientBoosting_Self:
        ...

    def n_iter_(self):
        ...


class HistGradientBoostingRegressor(RegressorMixin, BaseHistGradientBoosting):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    is_categorical_: None | ndarray = ...
    validation_score_: ndarray = ...
    train_score_: ndarray = ...
    n_trees_per_iteration_: int = ...
    n_iter_: int = ...
    do_early_stopping_: bool = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        loss: Literal[
            "squared_error", "absolute_error", "poisson", "quantile", "squared_error"
        ] = "squared_error",
        *,
        quantile: None | Float = None,
        learning_rate: Float = 0.1,
        max_iter: Int = 100,
        max_leaf_nodes: None | int = 31,
        max_depth: None | int = None,
        min_samples_leaf: Int = 20,
        l2_regularization: Float = 0.0,
        max_bins: Int = 255,
        categorical_features: None | MatrixLike | ArrayLike = None,
        monotonic_cst: None | Mapping | ArrayLike = None,
        interaction_cst: None
        | Literal["pairwise", "no_interaction"]
        | Sequence[list[int] | tuple[int, ...] | set[int]] = None,
        warm_start: bool = False,
        early_stopping: Literal["auto", "auto"] | bool = "auto",
        scoring: None | str | Callable = "loss",
        validation_fraction: float | None | int = 0.1,
        n_iter_no_change: Int = 10,
        tol: Float = 1e-7,
        verbose: Int = 0,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...

    def predict(self, X: MatrixLike) -> ndarray:
        ...

    def staged_predict(self, X: MatrixLike):
        ...


class HistGradientBoostingClassifier(ClassifierMixin, BaseHistGradientBoosting):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    is_categorical_: None | ndarray = ...
    validation_score_: ndarray = ...
    train_score_: ndarray = ...
    n_trees_per_iteration_: int = ...
    n_iter_: int = ...
    do_early_stopping_: bool = ...
    classes_: ndarray = ...

    # TODO(1.3): Remove "binary_crossentropy", "categorical_crossentropy", "auto"
    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        loss: Literal[
            "log_loss",
            "log_loss",
            "auto",
            "binary_crossentropy",
            "categorical_crossentropy",
        ] = "log_loss",
        *,
        learning_rate: Float = 0.1,
        max_iter: Int = 100,
        max_leaf_nodes: None | int = 31,
        max_depth: None | int = None,
        min_samples_leaf: Int = 20,
        l2_regularization: Float = 0.0,
        max_bins: Int = 255,
        categorical_features: None | MatrixLike | ArrayLike = None,
        monotonic_cst: None | Mapping | ArrayLike = None,
        interaction_cst: None
        | Literal["pairwise", "no_interaction"]
        | Sequence[list[int] | tuple[int, ...] | set[int]] = None,
        warm_start: bool = False,
        early_stopping: Literal["auto", "auto"] | bool = "auto",
        scoring: None | str | Callable = "loss",
        validation_fraction: float | None | int = 0.1,
        n_iter_no_change: Int = 10,
        tol: Float = 1e-7,
        verbose: Int = 0,
        random_state: RandomState | None | Int = None,
        class_weight: None | Mapping | str = None,
    ) -> None:
        ...

    def predict(self, X: MatrixLike) -> ndarray:
        ...

    def staged_predict(self, X: MatrixLike):
        ...

    def predict_proba(self, X: MatrixLike) -> ndarray:
        ...

    def staged_predict_proba(self, X: MatrixLike):
        ...

    def decision_function(self, X: MatrixLike) -> ndarray:
        ...

    def staged_decision_function(self, X: MatrixLike):
        ...



from typing import Literal
from os import listdir as listdir, makedirs as makedirs, remove as remove
from ._base import (
    get_data_home as get_data_home,
    RemoteFileMetadata as RemoteFileMetadata,
    load_descr as load_descr,
)
from os.path import join as join, exists as exists, isdir as isdir
from joblib import Memory as Memory
from ..utils._bunch import Bunch
from .._typing import Float, Int

import logging

import numpy as np

logger = ...

# The original data can be found in:
# http://vis-www.cs.umass.edu/lfw/lfw.tgz
ARCHIVE = ...

# The original funneled data can be found in:
# http://vis-www.cs.umass.edu/lfw/lfw-funneled.tgz
FUNNELED_ARCHIVE = ...

# The original target data can be found in:
# http://vis-www.cs.umass.edu/lfw/pairsDevTrain.txt',
# http://vis-www.cs.umass.edu/lfw/pairsDevTest.txt',
# http://vis-www.cs.umass.edu/lfw/pairs.txt',
TARGETS = ...


def fetch_lfw_people(
    *,
    data_home: None | str = None,
    funneled: bool = True,
    resize: None | Float = 0.5,
    min_faces_per_person: Int = 0,
    color: bool = False,
    slice_: tuple[slice, ...] = ...,
    download_if_missing: bool = True,
    return_X_y: bool = False,
) -> Bunch | tuple[Bunch, tuple]:
    ...


def fetch_lfw_pairs(
    *,
    subset: Literal["train", "test", "10_folds", "train"] = "train",
    data_home: None | str = None,
    funneled: bool = True,
    resize: Float = 0.5,
    color: bool = False,
    slice_: tuple[slice, ...] = ...,
    download_if_missing: bool = True,
) -> Bunch:
    ...



from typing import Mapping
from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

def get_color_names() -> ArrayLike: ...
def get_color_dict() -> Mapping: ...

# This is used by color functions to translate user strings to colors
# For now, this is web colors, and all in hex. It will take some simple
# but annoying refactoring to deal with non-hex entries if we want them.

# Add the CSS colors, courtesy MIT-licensed code from Dave Eddy:
# github.com/bahamas10/css-color-names/blob/master/css-color-names.json

_color_dict: dict = ...



from typing import ClassVar, Literal, TypeVar
from ..utils.validation import (
    check_is_fitted as check_is_fitted,
    column_or_1d as column_or_1d,
)
from abc import abstractmethod
from ..base import ClassifierMixin, RegressorMixin, BaseEstimator
from ._base import _BaseHeterogeneousEnsemble
from ..exceptions import NotFittedError as NotFittedError
from ..utils._param_validation import StrOptions as StrOptions
from numpy import ndarray
from numbers import Integral as Integral
from ..utils.multiclass import (
    check_classification_targets as check_classification_targets,
)
from ..utils.metaestimators import available_if as available_if
from ..base import TransformerMixin, clone as clone
from ..utils.parallel import delayed as delayed, Parallel as Parallel
from .._typing import MatrixLike, ArrayLike, Int
from ..utils import Bunch
from ..preprocessing import LabelEncoder

VotingClassifier_Self = TypeVar("VotingClassifier_Self", bound="VotingClassifier")
VotingRegressor_Self = TypeVar("VotingRegressor_Self", bound="VotingRegressor")


import numpy as np


class _BaseVoting(TransformerMixin, _BaseHeterogeneousEnsemble):

    _parameter_constraints: ClassVar[dict] = ...

    @abstractmethod
    def fit(self, X: ndarray, y: ndarray, sample_weight=None):
        ...

    def fit_transform(
        self, X: MatrixLike, y: None | ArrayLike = None, **fit_params
    ) -> ndarray:
        ...

    def n_features_in_(self):
        ...


class VotingClassifier(ClassifierMixin, _BaseVoting):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    classes_: ndarray = ...
    le_: LabelEncoder = ...
    named_estimators_: Bunch = ...
    estimators_: list[ClassifierMixin] = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        estimators: list[tuple[str, BaseEstimator]],
        *,
        voting: Literal["hard", "soft", "hard"] = "hard",
        weights: None | ArrayLike = None,
        n_jobs: None | Int = None,
        flatten_transform: bool = True,
        verbose: bool = False,
    ) -> None:
        ...

    def fit(
        self: VotingClassifier_Self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> VotingClassifier_Self:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ArrayLike:
        ...

    def predict_proba(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def transform(self, X: MatrixLike | ArrayLike):
        ...

    def get_feature_names_out(self, input_features: None | ArrayLike = None) -> ndarray:
        ...


class VotingRegressor(RegressorMixin, _BaseVoting):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    named_estimators_: Bunch = ...
    estimators_: list[RegressorMixin] = ...

    def __init__(
        self,
        estimators: list[tuple[str, BaseEstimator]],
        *,
        weights: None | ArrayLike = None,
        n_jobs: None | Int = None,
        verbose: bool = False,
    ) -> None:
        ...

    def fit(
        self: VotingRegressor_Self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> VotingRegressor_Self:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def transform(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def get_feature_names_out(self, input_features: None | ArrayLike = None) -> ndarray:
        ...



from numpy import dtype
from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014, Nicolas P. Rougier
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

import numpy as np
from ... import gloo
from .util import fetchcode
from .base_collection import BaseCollection
from ..shaders import ModularProgram
from ...util.event import EventEmitter

class Collection(BaseCollection):

    _gtypes: dict = ...

    def __init__(
        self,
        dtype: ArrayLike,
        itype: np.dtype | None,
        mode,
        vertex: str | tuple[str, ...],
        fragment: str | tuple[str, ...],
        program=None,
        **kwargs: str,
    ): ...
    def view(self, transform, viewport=None): ...
    def __getitem__(self, key): ...
    def __setitem__(self, key, value): ...
    def draw(self, mode=None): ...

class CollectionView(object):
    def __init__(self, collection, transform=None, viewport=None): ...
    def __getitem__(self, key): ...
    def __setitem__(self, key, value): ...
    def draw(self): ...



from numpy.typing import ArrayLike
from collections.abc import Iterable
from .._typing import Scalar
from typing import Mapping
from ..classes.graph import Graph

import math
from bisect import bisect_left
from itertools import accumulate, combinations, product

from ..classes.graph import Graph
from ..utils import py_random_state

__all__ = [
    "geometric_edges",
    "geographical_threshold_graph",
    "navigable_small_world_graph",
    "random_geometric_graph",
    "soft_random_geometric_graph",
    "thresholded_random_geometric_graph",
    "waxman_graph",
]

def euclidean(x, y): ...
def geometric_edges(G: Graph, radius: Scalar, p: Scalar = 2) -> ArrayLike: ...
@py_random_state(5)
def random_geometric_graph(
    n: int | Iterable,
    radius: float,
    dim: int = 2,
    pos: Mapping | None = None,
    p: float = 2,
    seed=None,
) -> Graph: ...
@py_random_state(6)
def soft_random_geometric_graph(
    n: int | Iterable,
    radius: float,
    dim: int = 2,
    pos: Mapping | None = None,
    p: float = 2,
    p_dist=None,
    seed=None,
) -> Graph: ...
@py_random_state(7)
def geographical_threshold_graph(
    n: int | Iterable,
    theta: float,
    dim: int = 2,
    pos: Mapping | None = None,
    weight: Mapping | None = None,
    metric=None,
    p_dist=None,
    seed=None,
) -> Graph: ...
@py_random_state(6)
def waxman_graph(
    n: int | Iterable,
    beta: float = 0.4,
    alpha: float = 0.1,
    L: float | None = None,
    domain=...,
    metric=None,
    seed=None,
) -> Graph: ...
@py_random_state(5)
def navigable_small_world_graph(
    n: int, p: int = 1, q: int = 1, r: float = 2, dim: int = 2, seed=None
): ...
@py_random_state(7)
def thresholded_random_geometric_graph(
    n: int | Iterable,
    radius: float,
    theta: float,
    dim: int = 2,
    pos: Mapping | None = None,
    weight: Mapping | None = None,
    p=2,
    seed=None,
) -> Graph: ...



"""
This type stub file was generated by pyright.
"""

"""Token-related utilities"""
Token = ...
def generate_tokens(readline):
    """wrap generate_tokens to catch EOF errors"""
    ...

def line_at_cursor(cell, cursor_pos=...):
    """Return the line in a cell at a given cursor position

    Used for calling line-based APIs that don't support multi-line input, yet.

    Parameters
    ----------
    cell : str
        multiline block of text
    cursor_pos : integer
        the cursor position

    Returns
    -------
    (line, offset): (string, integer)
        The line with the current cursor, and the character offset of the start of the line.
    """
    ...

def token_at_cursor(cell, cursor_pos=...):
    """Get the token at a given cursor

    Used for introspection.

    Function calls are prioritized, so the token for the callable will be returned
    if the cursor is anywhere inside the call.

    Parameters
    ----------
    cell : unicode
        A block of Python code
    cursor_pos : int
        The location of the cursor in the block where the token should be found
    """
    ...




from typing import Any, ClassVar, TypeVar
from scipy import sparse as sparse
from ..exceptions import ConvergenceWarning as ConvergenceWarning
from numpy import ndarray
from ..utils.extmath import row_norms as row_norms
from ..utils._param_validation import Interval as Interval
from numbers import Integral as Integral, Real as Real
from .._config import config_context as config_context
from math import sqrt as sqrt
from ..base import (
    TransformerMixin,
    ClusterMixin,
    BaseEstimator,
    ClassNamePrefixFeaturesOutMixin,
)
from scipy.sparse import spmatrix
from ..metrics.pairwise import euclidean_distances as euclidean_distances
from .._typing import Float, Int, ArrayLike, MatrixLike
from ..metrics import pairwise_distances_argmin as pairwise_distances_argmin
from ..utils.validation import check_is_fitted as check_is_fitted
from . import AgglomerativeClustering as AgglomerativeClustering

Birch_Self = TypeVar("Birch_Self", bound="Birch")

# Authors: Manoj Kumar <manojkumarsivaraj334@gmail.com>
#          Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
#          Joel Nothman <joel.nothman@gmail.com>
# License: BSD 3 clause

import warnings
import numpy as np


class _CFNode:
    squared_norm_: ndarray = ...
    centroids_: ndarray = ...
    init_sq_norm_: ndarray = ...
    init_centroids_: ndarray = ...
    next_leaf_: _CFNode = ...
    prev_leaf_: _CFNode = ...
    subclusters_: list = ...

    def __init__(
        self,
        *,
        threshold: Float,
        branching_factor: Int,
        is_leaf: bool,
        n_features: Int,
        dtype,
    ) -> None:
        ...

    def append_subcluster(self, subcluster: _CFSubcluster) -> None:
        ...

    def update_split_subclusters(
        self,
        subcluster: _CFSubcluster,
        new_subcluster1: _CFSubcluster,
        new_subcluster2: _CFSubcluster,
    ) -> None:
        ...

    def insert_cf_subcluster(self, subcluster: _CFSubcluster) -> bool:
        ...


class _CFSubcluster:
    sq_norm_: ndarray = ...
    child_: _CFNode = ...
    centroid_: ndarray = ...
    squared_sum_: float = ...
    linear_sum_: ndarray = ...
    n_samples_: int = ...

    def __init__(self, *, linear_sum: None | ArrayLike = None) -> None:
        ...

    def update(self, subcluster: _CFSubcluster) -> None:
        ...

    def merge_subcluster(
        self, nominee_cluster: _CFSubcluster, threshold: float
    ) -> bool:
        ...

    def radius(self):
        ...


class Birch(
    ClassNamePrefixFeaturesOutMixin, ClusterMixin, TransformerMixin, BaseEstimator
):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    labels_: ndarray = ...
    subcluster_labels_: ndarray = ...
    subcluster_centers_: ndarray = ...
    dummy_leaf_: _CFNode = ...
    root_: _CFNode = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        threshold: Float = 0.5,
        branching_factor: Int = 50,
        n_clusters: None | int = 3,
        compute_labels: bool = True,
        copy: bool = True,
    ) -> None:
        ...

    def fit(self: Birch_Self, X: MatrixLike | ArrayLike, y: Any = None) -> Birch_Self:
        ...

    def partial_fit(
        self: Birch_Self, X: None | MatrixLike | ArrayLike = None, y: Any = None
    ) -> Birch_Self:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def transform(self, X: MatrixLike | ArrayLike) -> ndarray | spmatrix:
        ...



from typing import Any, Callable, TypeVar

from django.contrib import admin
from django.core.handlers.wsgi import WSGIRequest
from django.http.response import HttpResponse

_F = TypeVar("_F", bound=Callable[..., Any])

def csrf_protect_m(func: _F) -> _F: ...
def sensitive_post_parameters_m(func: _F) -> _F: ...

class GroupAdmin(admin.ModelAdmin[Any]): ...

class UserAdmin(admin.ModelAdmin[Any]):
    change_user_password_template: Any = ...
    add_fieldsets: Any = ...
    add_form: Any = ...
    change_password_form: Any = ...
    def user_change_password(
        self, request: WSGIRequest, id: str, form_url: str = ...
    ) -> HttpResponse: ...



from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from collections import OrderedDict
import logging
import re

import numpy as np

from ...util.eq import eq
from ...util import logger
from . import parsing
from .shader_object import ShaderObject
from .variable import Variable, Varying
from .expression import TextExpression, FunctionCall

class Function(ShaderObject):
    def __init__(self, code, dependencies=None): ...
    def __setitem__(self, key, val): ...
    def __getitem__(self, key): ...
    def __call__(self, *args): ...
    def __contains__(self, key): ...

    # Public API methods

    @property
    def signature(self): ...
    @property
    def name(self): ...
    @property
    def args(self): ...
    @property
    def rtype(self): ...
    @property
    def code(self): ...
    @code.setter
    def code(self, code): ...
    @property
    def template_vars(self): ...
    def static_names(self): ...
    def replace(self, str1: str, str2: str): ...

    # Private methods

    def _parse_template_vars(self): ...
    def _get_replaced_code(self, names, version, shader): ...
    def definition(self, names, version, shader): ...
    def expression(self, names): ...
    def _clean_code(self, code): ...
    def __repr__(self): ...

class MainFunction(Function):
    def __init__(self, shader_type, *args, **kwargs): ...
    @property
    def signature(self): ...
    @property
    def version_pragma(self): ...
    def definition(self, obj_names, version, shader): ...
    def static_names(self): ...
    def add_chain(self, var): ...
    def add_callback(self, hook, func): ...
    def remove_callback(self, hook, func): ...

class FunctionChain(Function):
    def __init__(self, name: str | None = None, funcs: ArrayLike = ...): ...
    @property
    def functions(self): ...
    @functions.setter
    def functions(self, funcs): ...
    @property
    def signature(self): ...
    def _update(self): ...
    @property
    def code(self): ...
    @code.setter
    def code(self, c): ...
    @property
    def template_vars(self): ...
    def append(self, function, update=True): ...
    def __setitem__(self, index, func): ...
    def __getitem__(self, k): ...
    def insert(self, index, function, update=True): ...
    def remove(self, function, update=True): ...
    def definition(self, obj_names, version, shader): ...
    def static_names(self): ...
    def __repr__(self): ...

class StatementList(ShaderObject):
    def __init__(self): ...
    def add(self, item, position=5): ...
    def remove(self, item): ...
    def expression(self, obj_names): ...



from typing import Callable, ClassVar, Literal, TypeVar
from warnings import warn as warn
from numpy.random import RandomState
from ..decomposition import PCA as PCA
from ..utils.validation import (
    check_is_fitted as check_is_fitted,
    check_array as check_array,
)
from ..exceptions import ConvergenceWarning as ConvergenceWarning
from numpy import ndarray
from ..utils.extmath import softmax as softmax
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from scipy.optimize import minimize as minimize
from numbers import Integral as Integral, Real as Real
from ..utils.multiclass import (
    check_classification_targets as check_classification_targets,
)
from ..base import BaseEstimator, TransformerMixin, ClassNamePrefixFeaturesOutMixin
from ..utils.random import check_random_state as check_random_state
from .._typing import Int, MatrixLike, Float, ArrayLike
from ..metrics import pairwise_distances as pairwise_distances
from ..preprocessing import LabelEncoder as LabelEncoder

NeighborhoodComponentsAnalysis_Self = TypeVar(
    "NeighborhoodComponentsAnalysis_Self", bound="NeighborhoodComponentsAnalysis"
)

import numpy as np
import sys
import time


class NeighborhoodComponentsAnalysis(
    ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator
):
    feature_names_in_: ndarray = ...
    random_state_: RandomState = ...
    n_iter_: int = ...
    n_features_in_: int = ...
    components_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_components: None | Int = None,
        *,
        init: Literal["auto", "pca", "lda", "identity", "random", "auto"]
        | MatrixLike = "auto",
        warm_start: bool = False,
        max_iter: Int = 50,
        tol: Float = 1e-5,
        callback: None | Callable = None,
        verbose: Int = 0,
        random_state: None | RandomState | int = None,
    ) -> None:
        ...

    def fit(
        self: NeighborhoodComponentsAnalysis_Self, X: MatrixLike, y: ArrayLike
    ) -> NeighborhoodComponentsAnalysis_Self:
        ...

    def transform(self, X: MatrixLike) -> ndarray:
        ...



import numpy as np
from matplotlib._typing import *
from .triangulation import Triangulation

class TriFinder:
    def __init__(self, triangulation: Triangulation) -> None: ...

class TrapezoidMapTriFinder(TriFinder):
    def __init__(self, triangulation: Triangulation) -> None: ...
    def __call__(self, x: ArrayLike, y: ArrayLike) -> np.ndarray: ...



from typing import Any

from django.db.backends.base.creation import (
    BaseDatabaseCreation as BaseDatabaseCreation,
)

class DatabaseCreation(BaseDatabaseCreation):
    def sql_table_creation_suffix(self) -> Any: ...



from typing import Callable, Type
from scipy import sparse as sparse
from numpy import ndarray, bool_
from ..utils.extmath import squared_norm as squared_norm
from .._loss.loss import BaseLoss
from .._typing import Float, MatrixLike, ArrayLike, Int
import numpy as np


class LinearModelLoss:
    def __init__(self, base_loss: BaseLoss, fit_intercept: bool) -> None:
        ...

    def init_zero_coef(self, X: ndarray, dtype: None | Type[Float] = None) -> ndarray:
        ...

    def weight_intercept(
        self, coef: MatrixLike | ArrayLike
    ) -> tuple[ndarray, Float] | tuple[ndarray, ndarray] | tuple[
        ndarray, float | ndarray
    ]:
        ...

    def weight_intercept_raw(
        self, coef: MatrixLike | ArrayLike, X: MatrixLike | ArrayLike
    ) -> tuple[ndarray, float | ndarray, ndarray]:
        ...

    def l2_penalty(self, weights: ndarray, l2_reg_strength: Float) -> Float:
        ...

    def loss(
        self,
        coef: MatrixLike | ArrayLike,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        sample_weight: None | MatrixLike = None,
        l2_reg_strength: Float = 0.0,
        n_threads: Int = 1,
        raw_prediction: None | MatrixLike | ArrayLike = None,
    ) -> Float:
        ...

    def loss_gradient(
        self,
        coef: MatrixLike | ArrayLike,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        sample_weight: None | MatrixLike = None,
        l2_reg_strength: Float = 0.0,
        n_threads: Int = 1,
        raw_prediction: None | MatrixLike | ArrayLike = None,
    ) -> tuple[float, ndarray] | tuple[Float, ndarray]:
        ...

    def gradient(
        self,
        coef: MatrixLike | ArrayLike,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        sample_weight: None | MatrixLike = None,
        l2_reg_strength: Float = 0.0,
        n_threads: Int = 1,
        raw_prediction: None | MatrixLike | ArrayLike = None,
    ) -> ndarray:
        ...

    def gradient_hessian(
        self,
        coef: MatrixLike | ArrayLike,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        sample_weight: None | MatrixLike = None,
        l2_reg_strength: Float = 0.0,
        n_threads: Int = 1,
        gradient_out: None | ArrayLike = None,
        hessian_out: None | ArrayLike = None,
        raw_prediction: None | MatrixLike | ArrayLike = None,
    ) -> tuple[ndarray, ndarray, bool_] | tuple[ndarray, ndarray, bool]:
        ...

    def gradient_hessian_product(
        self,
        coef: MatrixLike | ArrayLike,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        sample_weight: None | MatrixLike = None,
        l2_reg_strength: Float = 0.0,
        n_threads: Int = 1,
    ) -> tuple[ndarray, Callable]:
        ...



from skimage.future.graph.rag import RAG
from numpy.typing import NDArray
from typing import Callable
import numpy as np
import heapq

def _revalidate_node_edges(rag, node, heap_list): ...
def _rename_node(graph, node_id, copy_id): ...
def _invalidate_edge(graph, n1, n2): ...
def merge_hierarchical(
    labels: NDArray,
    rag: RAG,
    thresh: float,
    rag_copy: bool,
    in_place_merge: bool,
    merge_func: Callable,
    weight_func: Callable,
) -> NDArray: ...



from vispy.util.svg.color import Color
from vispy.color.color_array import ColorArray
from numpy.typing import NDArray

from .mesh import MeshVisual
import numpy as np
from numpy.linalg import norm
from ..util.transforms import rotate
from ..color import ColorArray

import collections

class TubeVisual(MeshVisual):
    def __init__(
        self,
        points: NDArray,
        radius: NDArray | float = 1.0,
        closed: bool = False,
        color: ColorArray | Color | str = "purple",
        tube_points: int = 8,
        shading: str | None = "smooth",
        vertex_colors: NDArray | None = None,
        face_colors: NDArray | None = None,
        mode: str = "triangles",
    ): ...

def _frenet_frames(points, closed): ...
